"""Test communicating with a the device API."""
from __future__ import annotations

from http import HTTPStatus
from typing import TYPE_CHECKING

import pytest
from httpx import ConnectTimeout

from devolo_plc_api.device_api.factoryreset_pb2 import FactoryResetStart
from devolo_plc_api.device_api.ledsettings_pb2 import LedSettingsGet, LedSettingsSetResponse
from devolo_plc_api.device_api.multiap_pb2 import WifiMultiApGetResponse
from devolo_plc_api.device_api.restart_pb2 import RestartResponse, UptimeGetResponse
from devolo_plc_api.device_api.support_pb2 import SupportInfoDump, SupportInfoDumpResponse
from devolo_plc_api.device_api.updatefirmware_pb2 import UpdateFirmwareCheck, UpdateFirmwareStart
from devolo_plc_api.device_api.wifinetwork_pb2 import (
    WifiConnectedStationsGet,
    WifiGuestAccessGet,
    WifiGuestAccessSetResponse,
    WifiNeighborAPsGet,
    WifiRepeatedAPsGet,
    WifiRepeaterWpsClonePbcStart,
    WifiResult,
    WifiWpsPbcStart,
)
from devolo_plc_api.exceptions import DevicePasswordProtected, DeviceUnavailable, FeatureNotSupported

from . import DeviceType

if TYPE_CHECKING:
    from pytest_httpx import HTTPXMock

    from devolo_plc_api import Device
    from devolo_plc_api.device_api import ConnectedStationInfo, DeviceApi, NeighborAPInfo, RepeatedAPInfo, SupportInfoItem


class TestDeviceApi:
    """Test devolo_plc_api.device_api.deviceapi.DeviceApi class."""

    @pytest.mark.parametrize("feature", ["[]"])
    def test_unsupported_feature(self, device_api: DeviceApi):
        """Test raising on using unsupported feature."""
        with pytest.raises(FeatureNotSupported):
            device_api.get_led_setting()

    @pytest.mark.parametrize("feature", [""])
    def test_feature(self, device_api: DeviceApi):
        """Test list of default features."""
        assert device_api.features == ["reset", "update", "led", "intmtg"]

    @pytest.mark.asyncio()
    @pytest.mark.parametrize("device_type", [DeviceType.PLC])
    @pytest.mark.usefixtures("block_communication", "service_browser")
    async def test_wrong_password_type(self, httpx_mock: HTTPXMock, mock_device: Device):
        """Test using different password hash if original password failed."""
        await mock_device.async_connect()
        assert mock_device.device
        mock_device.password = "password"

        httpx_mock.add_response(status_code=HTTPStatus.UNAUTHORIZED)
        with pytest.raises(DevicePasswordProtected):
            await mock_device.device.async_get_wifi_connected_station()
            assert mock_device.device.password == "5e884898da28047151d0e56f8dc6292773603d0d6aabbdd62a11ef721d1542d8"

        httpx_mock.add_response(status_code=HTTPStatus.UNAUTHORIZED)
        with pytest.raises(DevicePasswordProtected):
            await mock_device.device.async_get_wifi_connected_station()
            assert mock_device.device.password == "113459eb7bb31bddee85ade5230d6ad5d8b2fb52879e00a84ff6ae1067a210d3"

        await mock_device.async_disconnect()

    @pytest.mark.asyncio()
    @pytest.mark.parametrize("feature", ["led"])
    async def test_async_get_led_setting(self, device_api: DeviceApi, httpx_mock: HTTPXMock):
        """Test getting LED settings asynchronously."""
        led_setting_get = LedSettingsGet(state=LedSettingsGet.LED_ON)
        httpx_mock.add_response(content=led_setting_get.SerializeToString())
        assert await device_api.async_get_led_setting()

    @pytest.mark.parametrize("feature", ["led"])
    def test_get_led_setting(self, device_api: DeviceApi, httpx_mock: HTTPXMock):
        """Test getting LED settings synchronously."""
        led_setting_get = LedSettingsGet(state=LedSettingsGet.LED_ON)
        httpx_mock.add_response(content=led_setting_get.SerializeToString())
        assert device_api.get_led_setting()

    @pytest.mark.asyncio()
    @pytest.mark.parametrize("feature", ["led"])
    async def test_async_set_led_setting(self, device_api: DeviceApi, httpx_mock: HTTPXMock):
        """Test setting LED settings asynchronously."""
        led_setting_set = LedSettingsSetResponse()
        httpx_mock.add_response(content=led_setting_set.SerializeToString())
        assert await device_api.async_set_led_setting(enable=True)

    @pytest.mark.parametrize("feature", ["led"])
    def test_set_led_setting(self, device_api: DeviceApi, httpx_mock: HTTPXMock):
        """Test setting LED settings synchronously."""
        led_setting_set = LedSettingsSetResponse()
        httpx_mock.add_response(content=led_setting_set.SerializeToString())
        assert device_api.set_led_setting(enable=True)

    @pytest.mark.asyncio()
    @pytest.mark.parametrize("feature", ["multiap"])
    async def test_async_get_wifi_multi_ap(self, device_api: DeviceApi, httpx_mock: HTTPXMock):
        """Test setting LED settings asynchronously."""
        multi_ap_details = WifiMultiApGetResponse(enabled=True)
        httpx_mock.add_response(content=multi_ap_details.SerializeToString())
        details = await device_api.async_get_wifi_multi_ap()
        assert details.enabled

    @pytest.mark.parametrize("feature", ["multiap"])
    def test_get_wifi_multi_ap(self, device_api: DeviceApi, httpx_mock: HTTPXMock):
        """Test setting LED settings synchronously."""
        multi_ap_details = WifiMultiApGetResponse(enabled=True)
        httpx_mock.add_response(content=multi_ap_details.SerializeToString())
        details = device_api.get_wifi_multi_ap()
        assert details.enabled

    @pytest.mark.asyncio()
    @pytest.mark.parametrize("feature", ["repeater0"])
    async def test_async_get_wifi_repeated_access_points(
        self, device_api: DeviceApi, httpx_mock: HTTPXMock, repeated_ap: RepeatedAPInfo
    ):
        """Test getting AP settings asynchronously."""
        wifi_repeated_accesspoints_get = WifiRepeatedAPsGet(repeated_aps=[repeated_ap])
        httpx_mock.add_response(content=wifi_repeated_accesspoints_get.SerializeToString())
        wifi_repeated_access_points = await device_api.async_get_wifi_repeated_access_points()
        assert wifi_repeated_access_points == [repeated_ap]

    @pytest.mark.parametrize("feature", ["repeater0"])
    def test_get_wifi_repeated_access_points(self, device_api: DeviceApi, httpx_mock: HTTPXMock, repeated_ap: RepeatedAPInfo):
        """Test getting AP settings synchronously."""
        wifi_repeated_accesspoints_get = WifiRepeatedAPsGet(repeated_aps=[repeated_ap])
        httpx_mock.add_response(content=wifi_repeated_accesspoints_get.SerializeToString())
        wifi_repeated_access_points = device_api.get_wifi_repeated_access_points()
        assert wifi_repeated_access_points == [repeated_ap]

    @pytest.mark.asyncio()
    @pytest.mark.parametrize("feature", ["repeater0"])
    async def test_async_start_wps_clone(self, device_api: DeviceApi, httpx_mock: HTTPXMock):
        """Test starting WPS clone mode asynchronously."""
        wps = WifiRepeaterWpsClonePbcStart()
        httpx_mock.add_response(content=wps.SerializeToString())
        assert await device_api.async_start_wps_clone()

    @pytest.mark.parametrize("feature", ["repeater0"])
    def test_start_wps_clone(self, device_api: DeviceApi, httpx_mock: HTTPXMock):
        """Test starting WPS clone mode synchronously."""
        wps = WifiRepeaterWpsClonePbcStart()
        httpx_mock.add_response(content=wps.SerializeToString())
        assert device_api.start_wps_clone()

    @pytest.mark.parametrize("feature", ["reset"])
    async def test_async_factory_reset(self, device_api: DeviceApi, httpx_mock: HTTPXMock):
        """Test factory reset asynchronously."""
        reset = FactoryResetStart()
        httpx_mock.add_response(content=reset.SerializeToString())
        assert await device_api.async_factory_reset()

    @pytest.mark.parametrize("feature", ["reset"])
    def test_factory_reset(self, device_api: DeviceApi, httpx_mock: HTTPXMock):
        """Test factory reset synchronously."""
        reset = FactoryResetStart()
        httpx_mock.add_response(content=reset.SerializeToString())
        assert device_api.factory_reset()

    @pytest.mark.asyncio()
    @pytest.mark.parametrize("feature", ["restart"])
    async def test_async_restart(self, device_api: DeviceApi, httpx_mock: HTTPXMock):
        """Test restarting a device asynchronously."""
        restart = RestartResponse()
        httpx_mock.add_response(content=restart.SerializeToString())
        assert await device_api.async_restart()

    @pytest.mark.parametrize("feature", ["restart"])
    def test_restart(self, device_api: DeviceApi, httpx_mock: HTTPXMock):
        """Test restarting a device synchronously."""
        restart = RestartResponse()
        httpx_mock.add_response(content=restart.SerializeToString())
        assert device_api.restart()

    @pytest.mark.asyncio()
    @pytest.mark.parametrize("feature", ["restart"])
    async def test_async_uptime(self, device_api: DeviceApi, httpx_mock: HTTPXMock, runtime: int):
        """Test getting a device's update asynchronously."""
        uptime = UptimeGetResponse(uptime=runtime)
        httpx_mock.add_response(content=uptime.SerializeToString())
        assert await device_api.async_uptime() == runtime

    @pytest.mark.parametrize("feature", ["restart"])
    def test_uptime(self, device_api: DeviceApi, httpx_mock: HTTPXMock, runtime: int):
        """Test getting a device's update synchronously."""
        uptime = UptimeGetResponse(uptime=runtime)
        httpx_mock.add_response(content=uptime.SerializeToString())
        assert device_api.uptime() == runtime

    @pytest.mark.asyncio()
    @pytest.mark.parametrize("feature", ["support"])
    async def test_async_get_support_info(self, device_api: DeviceApi, httpx_mock: HTTPXMock, support_item: SupportInfoItem):
        """Test getting a device's support information asynchronously."""
        support_info = SupportInfoDumpResponse(info=SupportInfoDump(items=[support_item]))
        httpx_mock.add_response(content=support_info.SerializeToString())
        assert await device_api.async_get_support_info() == support_info.info

    @pytest.mark.parametrize("feature", ["support"])
    def test_get_support_info(self, device_api: DeviceApi, httpx_mock: HTTPXMock, support_item: SupportInfoItem):
        """Test getting a device's support information synchronously."""
        support_info = SupportInfoDumpResponse(info=SupportInfoDump(items=[support_item]))
        httpx_mock.add_response(content=support_info.SerializeToString())
        assert device_api.get_support_info() == support_info.info

    @pytest.mark.asyncio()
    @pytest.mark.parametrize("feature", ["update"])
    async def test_async_check_firmware_available(
        self, device_api: DeviceApi, httpx_mock: HTTPXMock, firmware_update: UpdateFirmwareCheck
    ):
        """Test checking for firmware updates asynchronously."""
        httpx_mock.add_response(content=firmware_update.SerializeToString())
        firmware = await device_api.async_check_firmware_available()
        assert firmware == firmware_update

    @pytest.mark.parametrize("feature", ["update"])
    def test_check_firmware_available(
        self, device_api: DeviceApi, httpx_mock: HTTPXMock, firmware_update: UpdateFirmwareCheck
    ):
        """Test checking for firmware updates synchronously."""
        httpx_mock.add_response(content=firmware_update.SerializeToString())
        firmware = device_api.check_firmware_available()
        assert firmware == firmware_update

    @pytest.mark.asyncio()
    @pytest.mark.parametrize("feature", ["update"])
    async def test_async_start_firmware_update(self, device_api: DeviceApi, httpx_mock: HTTPXMock):
        """Test firmware update asynchronously."""
        firmware_update = UpdateFirmwareStart(result=UpdateFirmwareStart.UPDATE_STARTED)
        httpx_mock.add_response(content=firmware_update.SerializeToString())
        assert await device_api.async_start_firmware_update()

    @pytest.mark.parametrize("feature", ["update"])
    def test_start_firmware_update(self, device_api: DeviceApi, httpx_mock: HTTPXMock):
        """Test firmware update synchronously."""
        firmware_update = UpdateFirmwareStart(result=UpdateFirmwareStart.UPDATE_STARTED)
        httpx_mock.add_response(content=firmware_update.SerializeToString())
        assert device_api.start_firmware_update()

    @pytest.mark.asyncio()
    @pytest.mark.parametrize("feature", ["wifi1"])
    async def test_async_get_wifi_connected_station(
        self, device_api: DeviceApi, httpx_mock: HTTPXMock, connected_station: ConnectedStationInfo
    ):
        """Test getting connected wifi clients asynchronously."""
        wifi_connected_stations_get = WifiConnectedStationsGet(connected_stations=[connected_station])
        httpx_mock.add_response(content=wifi_connected_stations_get.SerializeToString())
        connected_stations = await device_api.async_get_wifi_connected_station()
        assert connected_stations == [connected_station]

    @pytest.mark.parametrize("feature", ["wifi1"])
    def test_get_wifi_connected_station(
        self, device_api: DeviceApi, httpx_mock: HTTPXMock, connected_station: ConnectedStationInfo
    ):
        """Test getting connected wifi clients synchronously."""
        wifi_connected_stations_get = WifiConnectedStationsGet(connected_stations=[connected_station])
        httpx_mock.add_response(content=wifi_connected_stations_get.SerializeToString())
        connected_stations = device_api.get_wifi_connected_station()
        assert connected_stations == [connected_station]

    @pytest.mark.asyncio()
    @pytest.mark.parametrize("feature", ["wifi1"])
    async def test_async_get_wifi_guest_access(self, device_api: DeviceApi, httpx_mock: HTTPXMock):
        """Test getting wifi guest access status asynchronously."""
        wifi_guest_access_get = WifiGuestAccessGet(enabled=True)
        httpx_mock.add_response(content=wifi_guest_access_get.SerializeToString())
        wifi_guest_access = await device_api.async_get_wifi_guest_access()
        assert wifi_guest_access == wifi_guest_access_get

    @pytest.mark.parametrize("feature", ["wifi1"])
    def test_get_wifi_guest_access(self, device_api: DeviceApi, httpx_mock: HTTPXMock):
        """Test getting wifi guest access status synchronously."""
        wifi_guest_access_get = WifiGuestAccessGet(enabled=True)
        httpx_mock.add_response(content=wifi_guest_access_get.SerializeToString())
        wifi_guest_access = device_api.get_wifi_guest_access()
        assert wifi_guest_access == wifi_guest_access_get

    @pytest.mark.asyncio()
    @pytest.mark.parametrize("feature", ["wifi1"])
    async def test_async_set_wifi_guest_access(self, device_api: DeviceApi, httpx_mock: HTTPXMock):
        """Test setting wifi guest access status asynchronously."""
        wifi_guest_access_set = WifiGuestAccessSetResponse(result=WifiResult.WIFI_SUCCESS)
        httpx_mock.add_response(content=wifi_guest_access_set.SerializeToString())
        assert await device_api.async_set_wifi_guest_access(enable=True)

    @pytest.mark.parametrize("feature", ["wifi1"])
    def test_set_wifi_guest_access(self, device_api: DeviceApi, httpx_mock: HTTPXMock):
        """Test setting wifi guest access status synchronously."""
        wifi_guest_access_set = WifiGuestAccessSetResponse(result=WifiResult.WIFI_SUCCESS)
        httpx_mock.add_response(content=wifi_guest_access_set.SerializeToString())
        assert device_api.set_wifi_guest_access(enable=True)

    @pytest.mark.asyncio()
    @pytest.mark.parametrize("feature", ["wifi1"])
    async def test_async_get_wifi_neighbor_access_points(
        self, device_api: DeviceApi, httpx_mock: HTTPXMock, neighbor_ap: NeighborAPInfo
    ):
        """Test getting neighboring wifi access points asynchronously."""
        wifi_neighbor_accesspoints_get = WifiNeighborAPsGet(neighbor_aps=[neighbor_ap])
        httpx_mock.add_response(content=wifi_neighbor_accesspoints_get.SerializeToString())
        wifi_neighbor_access_points = await device_api.async_get_wifi_neighbor_access_points()
        assert wifi_neighbor_access_points == [neighbor_ap]

    @pytest.mark.parametrize("feature", ["wifi1"])
    def test_get_wifi_neighbor_access_points(self, device_api: DeviceApi, httpx_mock: HTTPXMock, neighbor_ap: NeighborAPInfo):
        """Test getting neighboring wifi access points synchronously."""
        wifi_neighbor_accesspoints_get = WifiNeighborAPsGet(neighbor_aps=[neighbor_ap])
        httpx_mock.add_response(content=wifi_neighbor_accesspoints_get.SerializeToString())
        wifi_neighbor_access_points = device_api.get_wifi_neighbor_access_points()
        assert wifi_neighbor_access_points == [neighbor_ap]

    @pytest.mark.asyncio()
    @pytest.mark.parametrize("feature", ["wifi1"])
    async def test_async_start_wps(self, device_api: DeviceApi, httpx_mock: HTTPXMock):
        """Test starting WPS asynchronously."""
        wps = WifiWpsPbcStart(result=WifiResult.WIFI_SUCCESS)
        httpx_mock.add_response(content=wps.SerializeToString())
        assert await device_api.async_start_wps()

    @pytest.mark.parametrize("feature", ["wifi1"])
    def test_start_wps(self, device_api: DeviceApi, httpx_mock: HTTPXMock):
        """Test starting WPS synchronously."""
        wps = WifiWpsPbcStart(result=WifiResult.WIFI_SUCCESS)
        httpx_mock.add_response(content=wps.SerializeToString())
        assert device_api.start_wps()

    @pytest.mark.asyncio()
    @pytest.mark.parametrize("device_type", [DeviceType.PLC])
    @pytest.mark.usefixtures("block_communication", "service_browser")
    async def test_device_unavailable(self, httpx_mock: HTTPXMock, mock_device: Device):
        """Test device being unavailable."""
        await mock_device.async_connect()
        assert mock_device.device
        httpx_mock.add_exception(ConnectTimeout(""))
        with pytest.raises(DeviceUnavailable):
            await mock_device.device.async_get_wifi_connected_station()

    @pytest.mark.asyncio()
    @pytest.mark.parametrize("device_type", [DeviceType.PLC])
    @pytest.mark.usefixtures("block_communication", "service_browser")
    async def test_attribute_error(self, mock_device: Device):
        """Test raising on calling not existing method."""
        await mock_device.async_connect()
        assert mock_device.device
        with pytest.raises(AttributeError):
            mock_device.device.test()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: updatefirmware.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14updatefirmware.proto\x12\ndevice.api\"\xb9\x01\n\x13UpdateFirmwareCheck\x12\x36\n\x06result\x18\x01 \x01(\x0e\x32&.device.api.UpdateFirmwareCheck.Result\x12\x1c\n\x14new_firmware_version\x18\x02 \x01(\t\"L\n\x06Result\x12\x14\n\x10UPDATE_AVAILABLE\x10\x00\x12\x18\n\x14UPDATE_NOT_AVAILABLE\x10\x01\x12\x12\n\rUNKNOWN_ERROR\x10\xff\x01\"\x99\x01\n\x13UpdateFirmwareStart\x12\x36\n\x06result\x18\x01 \x01(\x0e\x32&.device.api.UpdateFirmwareStart.Result\"J\n\x06Result\x12\x12\n\x0eUPDATE_STARTED\x10\x00\x12\x18\n\x14UPDATE_NOT_AVAILABLE\x10\x01\x12\x12\n\rUNKNOWN_ERROR\x10\xff\x01\x42\x10\n\x06\x64\x65viceB\x06updateb\x06proto3')



_UPDATEFIRMWARECHECK = DESCRIPTOR.message_types_by_name['UpdateFirmwareCheck']
_UPDATEFIRMWARESTART = DESCRIPTOR.message_types_by_name['UpdateFirmwareStart']
_UPDATEFIRMWARECHECK_RESULT = _UPDATEFIRMWARECHECK.enum_types_by_name['Result']
_UPDATEFIRMWARESTART_RESULT = _UPDATEFIRMWARESTART.enum_types_by_name['Result']
UpdateFirmwareCheck = _reflection.GeneratedProtocolMessageType('UpdateFirmwareCheck', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEFIRMWARECHECK,
  '__module__' : 'updatefirmware_pb2'
  # @@protoc_insertion_point(class_scope:device.api.UpdateFirmwareCheck)
  })
_sym_db.RegisterMessage(UpdateFirmwareCheck)

UpdateFirmwareStart = _reflection.GeneratedProtocolMessageType('UpdateFirmwareStart', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEFIRMWARESTART,
  '__module__' : 'updatefirmware_pb2'
  # @@protoc_insertion_point(class_scope:device.api.UpdateFirmwareStart)
  })
_sym_db.RegisterMessage(UpdateFirmwareStart)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\006deviceB\006update'
  _UPDATEFIRMWARECHECK._serialized_start=37
  _UPDATEFIRMWARECHECK._serialized_end=222
  _UPDATEFIRMWARECHECK_RESULT._serialized_start=146
  _UPDATEFIRMWARECHECK_RESULT._serialized_end=222
  _UPDATEFIRMWARESTART._serialized_start=225
  _UPDATEFIRMWARESTART._serialized_end=378
  _UPDATEFIRMWARESTART_RESULT._serialized_start=304
  _UPDATEFIRMWARESTART_RESULT._serialized_end=378
# @@protoc_insertion_point(module_scope)



"""Test helper methods."""
from syrupy.assertion import SnapshotAssertion

from devolo_plc_api import wifi_qr_code
from devolo_plc_api.device_api.wifinetwork_pb2 import WPA_2, WifiGuestAccessGet


class TestHelpers:
    """Test devolo_plc_api.helpers."""

    def test_wifi_qr_code(self, snapshot: SnapshotAssertion):
        """Test creating a QR code."""
        wifi_guest_access = WifiGuestAccessGet(enabled=True, ssid='"Test"', key='"Test"', wpa=WPA_2)
        assert wifi_qr_code(wifi_guest_access) == snapshot



"""Discover devices in your network."""
from __future__ import annotations

import asyncio
import time
from ipaddress import ip_address
from typing import Any, cast

from ifaddr import get_adapters
from zeroconf import DNSQuestionType, ServiceBrowser, ServiceStateChange, Zeroconf

from devolo_plc_api.device import Device
from devolo_plc_api.device_api import SERVICE_TYPE


async def async_discover_network(timeout: float = 3) -> dict[str, Device]:
    """
    Discover all devices that expose the devolo device API via mDNS asynchronous.

    :return: Devices accessible via serial number.
    """
    devices: dict[str, Device] = {}

    def add(zeroconf: Zeroconf, service_type: str, name: str, state_change: ServiceStateChange) -> None:
        """React on state changes."""
        _add(zeroconf, service_type, name, state_change, devices=devices, timeout=timeout)

    browser = ServiceBrowser(Zeroconf(interfaces=_interfaces()), SERVICE_TYPE, [add], question_type=DNSQuestionType.QM)
    await asyncio.sleep(timeout)
    browser.cancel()
    return devices


def discover_network(timeout: float = 3) -> dict[str, Device]:
    """
    Discover devices that expose the devolo device API via mDNS synchronous.

    :return: Devices accessible via serial number.
    """
    devices: dict[str, Device] = {}

    def add(zeroconf: Zeroconf, service_type: str, name: str, state_change: ServiceStateChange) -> None:
        """React on state changes."""
        _add(zeroconf, service_type, name, state_change, devices=devices, timeout=timeout)

    browser = ServiceBrowser(Zeroconf(interfaces=_interfaces()), SERVICE_TYPE, [add], question_type=DNSQuestionType.QM)
    time.sleep(timeout)
    browser.cancel()
    return devices


def _add(zeroconf: Zeroconf, service_type: str, name: str, state_change: ServiceStateChange, **kwargs: Any) -> None:
    """Create a device object to each matching device."""
    if state_change is not ServiceStateChange.Added:
        return

    if (service_info := zeroconf.get_service_info(service_type, name, timeout=kwargs["timeout"] * 1000)) is None:
        return

    info = Device.info_from_service(service_info)
    if info is None or info.properties["MT"] in ("2600", "2601"):
        return  # Don't react on devolo Home Control central units

    devices: dict[str, Device] = kwargs["devices"]
    devices[info.properties["SN"]] = Device(ip=str(ip_address(info.address)), zeroconf_instance=zeroconf)


def _interfaces() -> list[str]:
    """Get IP addresses not being localhost."""
    interface: list[str] = []
    for adapter in get_adapters():
        interface.extend(cast(str, ip.ip) for ip in adapter.ips if ip.is_IPv4 and ip.ip != "127.0.0.1")
        interface.extend(cast(str, ip.ip[0]) for ip in adapter.ips if ip.is_IPv6 and ip.ip[0] != "::1")
    return interface



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class UpdateFirmwareCheck(google.protobuf.message.Message):
    """
    Check if new firmware available
    Use GET, no payload required
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class _Result:
        ValueType = typing.NewType('ValueType', builtins.int)
        V: typing_extensions.TypeAlias = ValueType
    class _ResultEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[UpdateFirmwareCheck._Result.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UPDATE_AVAILABLE: UpdateFirmwareCheck._Result.ValueType  # 0
        UPDATE_NOT_AVAILABLE: UpdateFirmwareCheck._Result.ValueType  # 1
        UNKNOWN_ERROR: UpdateFirmwareCheck._Result.ValueType  # 255
    class Result(_Result, metaclass=_ResultEnumTypeWrapper):
        pass

    UPDATE_AVAILABLE: UpdateFirmwareCheck.Result.ValueType  # 0
    UPDATE_NOT_AVAILABLE: UpdateFirmwareCheck.Result.ValueType  # 1
    UNKNOWN_ERROR: UpdateFirmwareCheck.Result.ValueType  # 255

    RESULT_FIELD_NUMBER: builtins.int
    NEW_FIRMWARE_VERSION_FIELD_NUMBER: builtins.int
    result: global___UpdateFirmwareCheck.Result.ValueType
    """contains the result of checking the firmware update"""

    new_firmware_version: typing.Text
    """Versionstring of the avaiable firmware"""

    def __init__(self,
        *,
        result: global___UpdateFirmwareCheck.Result.ValueType = ...,
        new_firmware_version: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["new_firmware_version",b"new_firmware_version","result",b"result"]) -> None: ...
global___UpdateFirmwareCheck = UpdateFirmwareCheck

class UpdateFirmwareStart(google.protobuf.message.Message):
    """
    Start firmware update
    Use GET, no payload required
    Important: The response does not tell you anything about the success of the update itself
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class _Result:
        ValueType = typing.NewType('ValueType', builtins.int)
        V: typing_extensions.TypeAlias = ValueType
    class _ResultEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[UpdateFirmwareStart._Result.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UPDATE_STARTED: UpdateFirmwareStart._Result.ValueType  # 0
        UPDATE_NOT_AVAILABLE: UpdateFirmwareStart._Result.ValueType  # 1
        UNKNOWN_ERROR: UpdateFirmwareStart._Result.ValueType  # 255
    class Result(_Result, metaclass=_ResultEnumTypeWrapper):
        pass

    UPDATE_STARTED: UpdateFirmwareStart.Result.ValueType  # 0
    UPDATE_NOT_AVAILABLE: UpdateFirmwareStart.Result.ValueType  # 1
    UNKNOWN_ERROR: UpdateFirmwareStart.Result.ValueType  # 255

    RESULT_FIELD_NUMBER: builtins.int
    result: global___UpdateFirmwareStart.Result.ValueType
    """contains the result of *triggering* the firmware update"""

    def __init__(self,
        *,
        result: global___UpdateFirmwareStart.Result.ValueType = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["result",b"result"]) -> None: ...
global___UpdateFirmwareStart = UpdateFirmwareStart



"""Unittests for devolo_plc_api."""
from __future__ import annotations

import json
from dataclasses import dataclass
from enum import Enum
from pathlib import Path
from typing import TYPE_CHECKING

from syrupy.extensions.amber import AmberSnapshotExtension

from devolo_plc_api.device_api import SERVICE_TYPE as DEVICE_API
from devolo_plc_api.plcnet_api import SERVICE_TYPE as PLCNET_API
from devolo_plc_api.zeroconf import ZeroconfServiceInfo

if TYPE_CHECKING:
    from syrupy.location import PyTestLocation


class DeviceType(Enum):
    """Different device types."""

    REPEATER = 1
    PLC = 2


class DifferentDirectoryExtension(AmberSnapshotExtension):
    """Extention for Syrupy to change the directory in which snapshots are stored."""

    @classmethod
    def dirname(cls: type[DifferentDirectoryExtension], *, test_location: PyTestLocation) -> str:
        """Store snapshots in `snapshots`rather than `__snapshots__`."""
        return str(Path(test_location.filepath).parent.joinpath("snapshots"))


@dataclass
class TestData:
    """Test data for a devolo device."""

    __test__ = False

    ip: str
    """IP address of a device."""

    hostname: str
    """Hostname of a device."""

    device_info: dict[str, ZeroconfServiceInfo]
    """Zeroconf info a device delivers."""


def load_test_data():
    """Load test data from file."""
    file = Path(__file__).parent / "test_data.json"
    with file.open("r") as handler:
        data = json.load(handler)
    device_info = {
        DEVICE_API: ZeroconfServiceInfo(hostname=data["hostname"], **data["device_info"][DEVICE_API]),
        PLCNET_API: ZeroconfServiceInfo(**data["device_info"][PLCNET_API]),
    }
    return TestData(ip=data["ip"], hostname=data["hostname"], device_info=device_info)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: wifinetwork.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11wifinetwork.proto\x12\ndevice.api\"E\n\x19WifiDeviceModeGetResponse\x12(\n\x04mode\x18\x01 \x01(\x0e\x32\x1a.device.api.WifiDeviceMode\"\xa9\x01\n\x11WifiDeviceModeSet\x12(\n\x04mode\x18\x01 \x01(\x0e\x32\x1a.device.api.WifiDeviceMode\x12\x30\n\tfh_params\x18\x02 \x01(\x0b\x32\x1d.device.api.WifiParametersSet\x12\x38\n\tbh_params\x18\x03 \x01(\x0b\x32%.device.api.WifiRepeaterParametersSet\"C\n\x19WifiDeviceModeSetResponse\x12&\n\x06result\x18\x01 \x01(\x0e\x32\x16.device.api.WifiResult\".\n\x11WifiParametersSet\x12\x0c\n\x04ssid\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\t\"C\n\x19WifiParametersSetResponse\x12&\n\x06result\x18\x01 \x01(\x0e\x32\x16.device.api.WifiResult\"w\n\x12WifiGuestAccessSet\x12\x0e\n\x06\x65nable\x18\x01 \x01(\x08\x12\x10\n\x08\x64uration\x18\x02 \x01(\r\x12\x0c\n\x04ssid\x18\x03 \x01(\t\x12\x0b\n\x03key\x18\x04 \x01(\t\x12$\n\x03wpa\x18\x05 \x01(\x0e\x32\x17.device.api.WifiWpaMode\"D\n\x1aWifiGuestAccessSetResponse\x12&\n\x06result\x18\x01 \x01(\x0e\x32\x16.device.api.WifiResult\"\x82\x01\n\x12WifiGuestAccessGet\x12\x0f\n\x07\x65nabled\x18\x01 \x01(\x08\x12\x1a\n\x12remaining_duration\x18\x02 \x01(\r\x12\x0c\n\x04ssid\x18\x03 \x01(\t\x12\x0b\n\x03key\x18\x04 \x01(\t\x12$\n\x03wpa\x18\x05 \x01(\x0e\x32\x17.device.api.WifiWpaMode\"\xe9\x01\n\x12WifiNeighborAPsGet\x12\x43\n\x0cneighbor_aps\x18\x01 \x03(\x0b\x32-.device.api.WifiNeighborAPsGet.NeighborAPInfo\x1a\x8d\x01\n\x0eNeighborAPInfo\x12\x13\n\x0bmac_address\x18\x01 \x01(\t\x12\x0c\n\x04ssid\x18\x02 \x01(\t\x12\"\n\x04\x62\x61nd\x18\x03 \x01(\x0e\x32\x14.device.api.WifiBand\x12\x0f\n\x07\x63hannel\x18\x04 \x01(\r\x12\x0e\n\x06signal\x18\x05 \x01(\x05\x12\x13\n\x0bsignal_bars\x18\x06 \x01(\x05\"\xf7\x01\n\x12WifiRepeatedAPsGet\x12\x43\n\x0crepeated_aps\x18\x01 \x03(\x0b\x32-.device.api.WifiRepeatedAPsGet.RepeatedAPInfo\x1a\x9b\x01\n\x0eRepeatedAPInfo\x12\x13\n\x0bmac_address\x18\x01 \x01(\t\x12\x0c\n\x04ssid\x18\x02 \x01(\t\x12\"\n\x04\x62\x61nd\x18\x03 \x01(\x0e\x32\x14.device.api.WifiBand\x12\x0f\n\x07\x63hannel\x18\x04 \x01(\r\x12\x0c\n\x04rate\x18\x05 \x01(\r\x12\x0e\n\x06signal\x18\x06 \x01(\x05\x12\x13\n\x0bsignal_bars\x18\x07 \x01(\x05\"\x90\x02\n\x18WifiConnectedStationsGet\x12U\n\x12\x63onnected_stations\x18\x01 \x03(\x0b\x32\x39.device.api.WifiConnectedStationsGet.ConnectedStationInfo\x1a\x9c\x01\n\x14\x43onnectedStationInfo\x12\x13\n\x0bmac_address\x18\x01 \x01(\t\x12)\n\x08vap_type\x18\x02 \x01(\x0e\x32\x17.device.api.WifiVAPType\x12\"\n\x04\x62\x61nd\x18\x03 \x01(\x0e\x32\x14.device.api.WifiBand\x12\x0f\n\x07rx_rate\x18\x04 \x01(\r\x12\x0f\n\x07tx_rate\x18\x05 \x01(\r\"u\n\x19WifiRepeaterParametersSet\x12\x0c\n\x04ssid\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\t\x12\x11\n\tcrossband\x18\x03 \x01(\x08\x12*\n\x0cprimary_band\x18\x04 \x01(\x0e\x32\x14.device.api.WifiBand\"K\n!WifiRepeaterParametersSetResponse\x12&\n\x06result\x18\x01 \x01(\x0e\x32\x16.device.api.WifiResult\"9\n\x0fWifiWpsPbcStart\x12&\n\x06result\x18\x01 \x01(\x0e\x32\x16.device.api.WifiResult\"F\n\x1cWifiRepeaterWpsClonePbcStart\x12&\n\x06result\x18\x01 \x01(\x0e\x32\x16.device.api.WifiResult*\xdb\x01\n\nWifiResult\x12\x10\n\x0cWIFI_SUCCESS\x10\x00\x12\x15\n\x11WIFI_INVALID_SSID\x10\x01\x12\x14\n\x10WIFI_INVALID_KEY\x10\x02\x12\x14\n\x10WIFI_IS_DISABLED\x10\x03\x12\x1e\n\x1aWIFI_INVALID_BACKHAUL_SSID\x10\x04\x12\x1d\n\x19WIFI_INVALID_BACKHAUL_KEY\x10\x05\x12 \n\x1cWIFI_UNSUPPORTED_DEVICE_MODE\x10\x06\x12\x17\n\x12WIFI_UNKNOWN_ERROR\x10\xff\x01*E\n\x08WifiBand\x12\x15\n\x11WIFI_BAND_UNKNOWN\x10\x00\x12\x10\n\x0cWIFI_BAND_2G\x10\x01\x12\x10\n\x0cWIFI_BAND_5G\x10\x02*f\n\x0bWifiVAPType\x12\x14\n\x10WIFI_VAP_UNKNOWN\x10\x00\x12\x14\n\x10WIFI_VAP_MAIN_AP\x10\x01\x12\x15\n\x11WIFI_VAP_GUEST_AP\x10\x02\x12\x14\n\x10WIFI_VAP_STATION\x10\x03*^\n\x0bWifiWpaMode\x12\x11\n\rWPA_NO_CHANGE\x10\x00\x12\x0c\n\x08WPA_NONE\x10\x01\x12\x0b\n\x07WPA_1_2\x10\x02\x12\t\n\x05WPA_2\x10\x03\x12\x0b\n\x07WPA_2_3\x10\x04\x12\t\n\x05WPA_3\x10\x05*?\n\x0eWifiDeviceMode\x12\x0f\n\x0bWDM_NO_CONF\x10\x00\x12\n\n\x06WDM_AP\x10\x02\x12\x10\n\x0cWDM_REPEATER\x10\x03\x42\x15\n\x06\x64\x65viceB\x0bwifinetworkb\x06proto3')

_WIFIRESULT = DESCRIPTOR.enum_types_by_name['WifiResult']
WifiResult = enum_type_wrapper.EnumTypeWrapper(_WIFIRESULT)
_WIFIBAND = DESCRIPTOR.enum_types_by_name['WifiBand']
WifiBand = enum_type_wrapper.EnumTypeWrapper(_WIFIBAND)
_WIFIVAPTYPE = DESCRIPTOR.enum_types_by_name['WifiVAPType']
WifiVAPType = enum_type_wrapper.EnumTypeWrapper(_WIFIVAPTYPE)
_WIFIWPAMODE = DESCRIPTOR.enum_types_by_name['WifiWpaMode']
WifiWpaMode = enum_type_wrapper.EnumTypeWrapper(_WIFIWPAMODE)
_WIFIDEVICEMODE = DESCRIPTOR.enum_types_by_name['WifiDeviceMode']
WifiDeviceMode = enum_type_wrapper.EnumTypeWrapper(_WIFIDEVICEMODE)
WIFI_SUCCESS = 0
WIFI_INVALID_SSID = 1
WIFI_INVALID_KEY = 2
WIFI_IS_DISABLED = 3
WIFI_INVALID_BACKHAUL_SSID = 4
WIFI_INVALID_BACKHAUL_KEY = 5
WIFI_UNSUPPORTED_DEVICE_MODE = 6
WIFI_UNKNOWN_ERROR = 255
WIFI_BAND_UNKNOWN = 0
WIFI_BAND_2G = 1
WIFI_BAND_5G = 2
WIFI_VAP_UNKNOWN = 0
WIFI_VAP_MAIN_AP = 1
WIFI_VAP_GUEST_AP = 2
WIFI_VAP_STATION = 3
WPA_NO_CHANGE = 0
WPA_NONE = 1
WPA_1_2 = 2
WPA_2 = 3
WPA_2_3 = 4
WPA_3 = 5
WDM_NO_CONF = 0
WDM_AP = 2
WDM_REPEATER = 3


_WIFIDEVICEMODEGETRESPONSE = DESCRIPTOR.message_types_by_name['WifiDeviceModeGetResponse']
_WIFIDEVICEMODESET = DESCRIPTOR.message_types_by_name['WifiDeviceModeSet']
_WIFIDEVICEMODESETRESPONSE = DESCRIPTOR.message_types_by_name['WifiDeviceModeSetResponse']
_WIFIPARAMETERSSET = DESCRIPTOR.message_types_by_name['WifiParametersSet']
_WIFIPARAMETERSSETRESPONSE = DESCRIPTOR.message_types_by_name['WifiParametersSetResponse']
_WIFIGUESTACCESSSET = DESCRIPTOR.message_types_by_name['WifiGuestAccessSet']
_WIFIGUESTACCESSSETRESPONSE = DESCRIPTOR.message_types_by_name['WifiGuestAccessSetResponse']
_WIFIGUESTACCESSGET = DESCRIPTOR.message_types_by_name['WifiGuestAccessGet']
_WIFINEIGHBORAPSGET = DESCRIPTOR.message_types_by_name['WifiNeighborAPsGet']
_WIFINEIGHBORAPSGET_NEIGHBORAPINFO = _WIFINEIGHBORAPSGET.nested_types_by_name['NeighborAPInfo']
_WIFIREPEATEDAPSGET = DESCRIPTOR.message_types_by_name['WifiRepeatedAPsGet']
_WIFIREPEATEDAPSGET_REPEATEDAPINFO = _WIFIREPEATEDAPSGET.nested_types_by_name['RepeatedAPInfo']
_WIFICONNECTEDSTATIONSGET = DESCRIPTOR.message_types_by_name['WifiConnectedStationsGet']
_WIFICONNECTEDSTATIONSGET_CONNECTEDSTATIONINFO = _WIFICONNECTEDSTATIONSGET.nested_types_by_name['ConnectedStationInfo']
_WIFIREPEATERPARAMETERSSET = DESCRIPTOR.message_types_by_name['WifiRepeaterParametersSet']
_WIFIREPEATERPARAMETERSSETRESPONSE = DESCRIPTOR.message_types_by_name['WifiRepeaterParametersSetResponse']
_WIFIWPSPBCSTART = DESCRIPTOR.message_types_by_name['WifiWpsPbcStart']
_WIFIREPEATERWPSCLONEPBCSTART = DESCRIPTOR.message_types_by_name['WifiRepeaterWpsClonePbcStart']
WifiDeviceModeGetResponse = _reflection.GeneratedProtocolMessageType('WifiDeviceModeGetResponse', (_message.Message,), {
  'DESCRIPTOR' : _WIFIDEVICEMODEGETRESPONSE,
  '__module__' : 'wifinetwork_pb2'
  # @@protoc_insertion_point(class_scope:device.api.WifiDeviceModeGetResponse)
  })
_sym_db.RegisterMessage(WifiDeviceModeGetResponse)

WifiDeviceModeSet = _reflection.GeneratedProtocolMessageType('WifiDeviceModeSet', (_message.Message,), {
  'DESCRIPTOR' : _WIFIDEVICEMODESET,
  '__module__' : 'wifinetwork_pb2'
  # @@protoc_insertion_point(class_scope:device.api.WifiDeviceModeSet)
  })
_sym_db.RegisterMessage(WifiDeviceModeSet)

WifiDeviceModeSetResponse = _reflection.GeneratedProtocolMessageType('WifiDeviceModeSetResponse', (_message.Message,), {
  'DESCRIPTOR' : _WIFIDEVICEMODESETRESPONSE,
  '__module__' : 'wifinetwork_pb2'
  # @@protoc_insertion_point(class_scope:device.api.WifiDeviceModeSetResponse)
  })
_sym_db.RegisterMessage(WifiDeviceModeSetResponse)

WifiParametersSet = _reflection.GeneratedProtocolMessageType('WifiParametersSet', (_message.Message,), {
  'DESCRIPTOR' : _WIFIPARAMETERSSET,
  '__module__' : 'wifinetwork_pb2'
  # @@protoc_insertion_point(class_scope:device.api.WifiParametersSet)
  })
_sym_db.RegisterMessage(WifiParametersSet)

WifiParametersSetResponse = _reflection.GeneratedProtocolMessageType('WifiParametersSetResponse', (_message.Message,), {
  'DESCRIPTOR' : _WIFIPARAMETERSSETRESPONSE,
  '__module__' : 'wifinetwork_pb2'
  # @@protoc_insertion_point(class_scope:device.api.WifiParametersSetResponse)
  })
_sym_db.RegisterMessage(WifiParametersSetResponse)

WifiGuestAccessSet = _reflection.GeneratedProtocolMessageType('WifiGuestAccessSet', (_message.Message,), {
  'DESCRIPTOR' : _WIFIGUESTACCESSSET,
  '__module__' : 'wifinetwork_pb2'
  # @@protoc_insertion_point(class_scope:device.api.WifiGuestAccessSet)
  })
_sym_db.RegisterMessage(WifiGuestAccessSet)

WifiGuestAccessSetResponse = _reflection.GeneratedProtocolMessageType('WifiGuestAccessSetResponse', (_message.Message,), {
  'DESCRIPTOR' : _WIFIGUESTACCESSSETRESPONSE,
  '__module__' : 'wifinetwork_pb2'
  # @@protoc_insertion_point(class_scope:device.api.WifiGuestAccessSetResponse)
  })
_sym_db.RegisterMessage(WifiGuestAccessSetResponse)

WifiGuestAccessGet = _reflection.GeneratedProtocolMessageType('WifiGuestAccessGet', (_message.Message,), {
  'DESCRIPTOR' : _WIFIGUESTACCESSGET,
  '__module__' : 'wifinetwork_pb2'
  # @@protoc_insertion_point(class_scope:device.api.WifiGuestAccessGet)
  })
_sym_db.RegisterMessage(WifiGuestAccessGet)

WifiNeighborAPsGet = _reflection.GeneratedProtocolMessageType('WifiNeighborAPsGet', (_message.Message,), {

  'NeighborAPInfo' : _reflection.GeneratedProtocolMessageType('NeighborAPInfo', (_message.Message,), {
    'DESCRIPTOR' : _WIFINEIGHBORAPSGET_NEIGHBORAPINFO,
    '__module__' : 'wifinetwork_pb2'
    # @@protoc_insertion_point(class_scope:device.api.WifiNeighborAPsGet.NeighborAPInfo)
    })
  ,
  'DESCRIPTOR' : _WIFINEIGHBORAPSGET,
  '__module__' : 'wifinetwork_pb2'
  # @@protoc_insertion_point(class_scope:device.api.WifiNeighborAPsGet)
  })
_sym_db.RegisterMessage(WifiNeighborAPsGet)
_sym_db.RegisterMessage(WifiNeighborAPsGet.NeighborAPInfo)

WifiRepeatedAPsGet = _reflection.GeneratedProtocolMessageType('WifiRepeatedAPsGet', (_message.Message,), {

  'RepeatedAPInfo' : _reflection.GeneratedProtocolMessageType('RepeatedAPInfo', (_message.Message,), {
    'DESCRIPTOR' : _WIFIREPEATEDAPSGET_REPEATEDAPINFO,
    '__module__' : 'wifinetwork_pb2'
    # @@protoc_insertion_point(class_scope:device.api.WifiRepeatedAPsGet.RepeatedAPInfo)
    })
  ,
  'DESCRIPTOR' : _WIFIREPEATEDAPSGET,
  '__module__' : 'wifinetwork_pb2'
  # @@protoc_insertion_point(class_scope:device.api.WifiRepeatedAPsGet)
  })
_sym_db.RegisterMessage(WifiRepeatedAPsGet)
_sym_db.RegisterMessage(WifiRepeatedAPsGet.RepeatedAPInfo)

WifiConnectedStationsGet = _reflection.GeneratedProtocolMessageType('WifiConnectedStationsGet', (_message.Message,), {

  'ConnectedStationInfo' : _reflection.GeneratedProtocolMessageType('ConnectedStationInfo', (_message.Message,), {
    'DESCRIPTOR' : _WIFICONNECTEDSTATIONSGET_CONNECTEDSTATIONINFO,
    '__module__' : 'wifinetwork_pb2'
    # @@protoc_insertion_point(class_scope:device.api.WifiConnectedStationsGet.ConnectedStationInfo)
    })
  ,
  'DESCRIPTOR' : _WIFICONNECTEDSTATIONSGET,
  '__module__' : 'wifinetwork_pb2'
  # @@protoc_insertion_point(class_scope:device.api.WifiConnectedStationsGet)
  })
_sym_db.RegisterMessage(WifiConnectedStationsGet)
_sym_db.RegisterMessage(WifiConnectedStationsGet.ConnectedStationInfo)

WifiRepeaterParametersSet = _reflection.GeneratedProtocolMessageType('WifiRepeaterParametersSet', (_message.Message,), {
  'DESCRIPTOR' : _WIFIREPEATERPARAMETERSSET,
  '__module__' : 'wifinetwork_pb2'
  # @@protoc_insertion_point(class_scope:device.api.WifiRepeaterParametersSet)
  })
_sym_db.RegisterMessage(WifiRepeaterParametersSet)

WifiRepeaterParametersSetResponse = _reflection.GeneratedProtocolMessageType('WifiRepeaterParametersSetResponse', (_message.Message,), {
  'DESCRIPTOR' : _WIFIREPEATERPARAMETERSSETRESPONSE,
  '__module__' : 'wifinetwork_pb2'
  # @@protoc_insertion_point(class_scope:device.api.WifiRepeaterParametersSetResponse)
  })
_sym_db.RegisterMessage(WifiRepeaterParametersSetResponse)

WifiWpsPbcStart = _reflection.GeneratedProtocolMessageType('WifiWpsPbcStart', (_message.Message,), {
  'DESCRIPTOR' : _WIFIWPSPBCSTART,
  '__module__' : 'wifinetwork_pb2'
  # @@protoc_insertion_point(class_scope:device.api.WifiWpsPbcStart)
  })
_sym_db.RegisterMessage(WifiWpsPbcStart)

WifiRepeaterWpsClonePbcStart = _reflection.GeneratedProtocolMessageType('WifiRepeaterWpsClonePbcStart', (_message.Message,), {
  'DESCRIPTOR' : _WIFIREPEATERWPSCLONEPBCSTART,
  '__module__' : 'wifinetwork_pb2'
  # @@protoc_insertion_point(class_scope:device.api.WifiRepeaterWpsClonePbcStart)
  })
_sym_db.RegisterMessage(WifiRepeaterWpsClonePbcStart)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\006deviceB\013wifinetwork'
  _WIFIRESULT._serialized_start=1875
  _WIFIRESULT._serialized_end=2094
  _WIFIBAND._serialized_start=2096
  _WIFIBAND._serialized_end=2165
  _WIFIVAPTYPE._serialized_start=2167
  _WIFIVAPTYPE._serialized_end=2269
  _WIFIWPAMODE._serialized_start=2271
  _WIFIWPAMODE._serialized_end=2365
  _WIFIDEVICEMODE._serialized_start=2367
  _WIFIDEVICEMODE._serialized_end=2430
  _WIFIDEVICEMODEGETRESPONSE._serialized_start=33
  _WIFIDEVICEMODEGETRESPONSE._serialized_end=102
  _WIFIDEVICEMODESET._serialized_start=105
  _WIFIDEVICEMODESET._serialized_end=274
  _WIFIDEVICEMODESETRESPONSE._serialized_start=276
  _WIFIDEVICEMODESETRESPONSE._serialized_end=343
  _WIFIPARAMETERSSET._serialized_start=345
  _WIFIPARAMETERSSET._serialized_end=391
  _WIFIPARAMETERSSETRESPONSE._serialized_start=393
  _WIFIPARAMETERSSETRESPONSE._serialized_end=460
  _WIFIGUESTACCESSSET._serialized_start=462
  _WIFIGUESTACCESSSET._serialized_end=581
  _WIFIGUESTACCESSSETRESPONSE._serialized_start=583
  _WIFIGUESTACCESSSETRESPONSE._serialized_end=651
  _WIFIGUESTACCESSGET._serialized_start=654
  _WIFIGUESTACCESSGET._serialized_end=784
  _WIFINEIGHBORAPSGET._serialized_start=787
  _WIFINEIGHBORAPSGET._serialized_end=1020
  _WIFINEIGHBORAPSGET_NEIGHBORAPINFO._serialized_start=879
  _WIFINEIGHBORAPSGET_NEIGHBORAPINFO._serialized_end=1020
  _WIFIREPEATEDAPSGET._serialized_start=1023
  _WIFIREPEATEDAPSGET._serialized_end=1270
  _WIFIREPEATEDAPSGET_REPEATEDAPINFO._serialized_start=1115
  _WIFIREPEATEDAPSGET_REPEATEDAPINFO._serialized_end=1270
  _WIFICONNECTEDSTATIONSGET._serialized_start=1273
  _WIFICONNECTEDSTATIONSGET._serialized_end=1545
  _WIFICONNECTEDSTATIONSGET_CONNECTEDSTATIONINFO._serialized_start=1389
  _WIFICONNECTEDSTATIONSGET_CONNECTEDSTATIONINFO._serialized_end=1545
  _WIFIREPEATERPARAMETERSSET._serialized_start=1547
  _WIFIREPEATERPARAMETERSSET._serialized_end=1664
  _WIFIREPEATERPARAMETERSSETRESPONSE._serialized_start=1666
  _WIFIREPEATERPARAMETERSSETRESPONSE._serialized_end=1741
  _WIFIWPSPBCSTART._serialized_start=1743
  _WIFIWPSPBCSTART._serialized_end=1800
  _WIFIREPEATERWPSCLONEPBCSTART._serialized_start=1802
  _WIFIREPEATERWPSCLONEPBCSTART._serialized_end=1872
# @@protoc_insertion_point(module_scope)



"""Representation of your devolo device."""
from __future__ import annotations

import asyncio
import logging
from contextlib import suppress
from datetime import date
from ipaddress import ip_address, ip_network
from struct import unpack_from
from typing import TYPE_CHECKING, cast

from httpx import AsyncClient
from ifaddr import get_adapters
from zeroconf import DNSQuestionType, ServiceInfo, ServiceStateChange, Zeroconf
from zeroconf.asyncio import AsyncServiceBrowser, AsyncServiceInfo, AsyncZeroconf

from .device_api import SERVICE_TYPE as DEVICEAPI, DeviceApi
from .exceptions import DeviceNotFound
from .plcnet_api import DEVICES_WITHOUT_PLCNET, SERVICE_TYPE as PLCNETAPI, PlcNetApi
from .zeroconf import ZeroconfServiceInfo

if TYPE_CHECKING:
    from types import TracebackType

    from typing_extensions import Self


class Device:
    """
    Representing object for your devolo PLC device. It stores all properties and functionalities discovered during setup.

    :param ip: IP address of the device to communicate with.
    :param plcnetapi: Reuse externally gathered data for the plcnet API
    :param deviceapi: Reuse externally gathered data for the device API
    :param zeroconf_instance: Zeroconf instance to be potentially reused.
    """

    MDNS_TIMEOUT = 300

    def __init__(
        self,
        ip: str,
        zeroconf_instance: AsyncZeroconf | Zeroconf | None = None,
    ) -> None:
        """Initialize the device."""
        self.ip = ip
        self.mac = ""
        self.mt_number = "0"
        self.product = ""
        self.technology = ""
        self.serial_number = "0"

        self.device: DeviceApi | None = None
        self.plcnet: PlcNetApi | None = None

        self._background_tasks: set[asyncio.Task] = set()
        self._browser: AsyncServiceBrowser | None = None
        self._connected = False
        self._info: dict[str, ZeroconfServiceInfo] = {PLCNETAPI: ZeroconfServiceInfo(), DEVICEAPI: ZeroconfServiceInfo()}
        self._logger = logging.getLogger(f"{self.__class__.__module__}.{self.__class__.__name__}")
        self._multicast = False
        self._password = ""
        self._session_instance: AsyncClient | None = None
        self._zeroconf_instance = zeroconf_instance
        logging.captureWarnings(capture=True)

        self._loop: asyncio.AbstractEventLoop
        self._session: AsyncClient
        self._zeroconf: AsyncZeroconf

    def __del__(self) -> None:
        """Warn user, if the connection was not properly closed."""
        if self._connected and self._session_instance is None:
            self._logger.warning("Please disconnect properly from the device.")

    async def __aenter__(self) -> Self:
        """Connect to a device asynchronously when entering a context manager."""
        await self.async_connect()
        return self

    async def __aexit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None:
        """Disconnect to a device asynchronously when entering a context manager."""
        await self.async_disconnect()

    def __enter__(self) -> Self:
        """Connect to a device synchronously when leaving a context manager."""
        self.connect()
        return self

    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None:
        """Disconnect to a device synchronously when leaving a context manager."""
        self.disconnect()

    @property
    def firmware_date(self) -> date:
        """Date the firmware was built."""
        return date.fromisoformat(self._info[DEVICEAPI].properties.get("FirmwareDate", "1970-01-01")[:10])

    @property
    def firmware_version(self) -> str:
        """Firmware version currently installed."""
        return self._info[DEVICEAPI].properties.get("FirmwareVersion", "")

    @property
    def hostname(self) -> str:
        """mDNS hostname of the device."""  # noqa: D403
        return self._info[DEVICEAPI].hostname

    @property
    def password(self) -> str:
        """The currently set device password."""
        return self._password

    @password.setter
    def password(self, password: str) -> None:
        """Change the currently set device password."""
        self._password = password
        if self.device:
            self.device.password = password
        if self.plcnet:
            self.plcnet.password = password

    async def async_connect(self, session_instance: AsyncClient | None = None) -> None:
        """
        Connect to a device asynchronous.

        :param: session_instance: Session client instance to be potentially reused.
        """
        self._session_instance = session_instance
        self._session = self._session_instance or AsyncClient()
        if not self._zeroconf_instance:
            self._zeroconf = AsyncZeroconf(interfaces=await self._get_relevant_interfaces())
        elif isinstance(self._zeroconf_instance, Zeroconf):
            self._zeroconf = AsyncZeroconf(zc=self._zeroconf_instance)
        else:
            self._zeroconf = self._zeroconf_instance
        await self._get_zeroconf_info()
        if not self._info[DEVICEAPI].properties and not self._info[PLCNETAPI].properties:
            await self._retry_zeroconf_info()
        if not self.device and not self.plcnet:
            raise DeviceNotFound(self.ip)
        self._connected = True

    def connect(self) -> None:
        """Connect to a device synchronous."""
        self._loop = asyncio.get_event_loop()
        self._loop.run_until_complete(self.async_connect())

    async def async_disconnect(self) -> None:
        """Disconnect from a device asynchronous."""
        if self._connected:
            if self._browser:
                await self._browser.async_cancel()
            if not self._zeroconf_instance:
                await self._zeroconf.async_close()
            if not self._session_instance:
                await self._session.aclose()
            self._connected = False

    def disconnect(self) -> None:
        """Disconnect from a device synchronous."""
        self._loop.run_until_complete(self.async_disconnect())

    async def _get_relevant_interfaces(self) -> list[str]:
        """Get the IP address of the relevant interface to reduce traffic."""
        interface: list[str] = []
        for adapter in get_adapters():
            interface.extend(
                cast(str, ip.ip)
                for ip in adapter.ips
                if ip.is_IPv4 and ip_address(self.ip) in ip_network(f"{ip.ip}/{ip.network_prefix}", strict=False)
            )
            interface.extend(
                cast(str, ip.ip[0])
                for ip in adapter.ips
                if ip.is_IPv6 and ip_address(self.ip) in ip_network(f"{ip.ip[0]}/{ip.network_prefix}", strict=False)
            )
        return interface

    async def _get_device_info(self) -> None:
        """Get information from the devolo Device API."""
        service_type = DEVICEAPI
        if self._info[service_type].properties:
            self.mt_number = self._info[service_type].properties.get("MT", "0")
            self.product = self._info[service_type].properties.get("Product", "")
            self.serial_number = self._info[service_type].properties["SN"]
            self.device = DeviceApi(
                ip=str(ip_address(self._info[service_type].address)),
                session=self._session,
                info=self._info[service_type],
            )
            self.device.password = self.password

    async def _get_plcnet_info(self) -> None:
        """Get information from the devolo PlcNet API."""
        service_type = PLCNETAPI
        if self._info[service_type].properties:
            self.mac = self._info[service_type].properties["PlcMacAddress"]
            self.technology = self._info[service_type].properties.get("PlcTechnology", "")
            self.plcnet = PlcNetApi(
                ip=str(ip_address(self._info[service_type].address)),
                session=self._session,
                info=self._info[service_type],
            )
            self.plcnet.password = self.password

    async def _get_zeroconf_info(self) -> None:
        """Browse for the desired mDNS service types and query them."""
        service_types = [DEVICEAPI, PLCNETAPI]
        counter = 0
        self._logger.debug("Browsing for %s", service_types)
        addr = None if self._multicast else self.ip
        question_type = DNSQuestionType.QM if self._multicast else DNSQuestionType.QU
        if self._browser:
            await self._browser.async_cancel()
            self._browser = None
        self._browser = AsyncServiceBrowser(
            zeroconf=self._zeroconf.zeroconf,
            type_=service_types,
            handlers=[self._state_change],
            addr=addr,
            question_type=question_type,
        )
        while (
            not self._info[DEVICEAPI].properties
            or not self._info[PLCNETAPI].properties
            and self.mt_number not in DEVICES_WITHOUT_PLCNET
        ) and counter < self.MDNS_TIMEOUT:
            counter += 1
            await asyncio.sleep(0.01)

    async def _retry_zeroconf_info(self) -> None:
        """Retry getting the zeroconf info using multicast."""
        self._logger.debug("Having trouble getting results via unicast messages. Switching to multicast for this device.")
        self._multicast = True
        await self._get_zeroconf_info()

    def _state_change(self, zeroconf: Zeroconf, service_type: str, name: str, state_change: ServiceStateChange) -> None:
        """Evaluate the query result."""
        if state_change == ServiceStateChange.Removed:
            return
        task = asyncio.create_task(self._get_service_info(zeroconf, service_type, name))
        self._background_tasks.add(task)
        task.add_done_callback(self._background_tasks.remove)

    async def _get_service_info(self, zeroconf: Zeroconf, service_type: str, name: str) -> None:
        """Get service information, if IP matches."""
        service_info = AsyncServiceInfo(service_type, name)
        update = {
            DEVICEAPI: self._get_device_info,
            PLCNETAPI: self._get_plcnet_info,
        }
        with suppress(RuntimeError):
            if not self._multicast:
                await service_info.async_request(zeroconf, timeout=1000, question_type=DNSQuestionType.QU, addr=self.ip)
            else:
                await service_info.async_request(zeroconf, timeout=1000, question_type=DNSQuestionType.QM)

        if not service_info.addresses or self.ip not in service_info.parsed_addresses():
            return  # No need to continue, if there are no relevant service information

        self._logger.debug("Updating service info of %s for %s", service_type, service_info.server_key)
        if info := self.info_from_service(service_info):
            self._info[service_type] = info
            await update[service_type]()

    @staticmethod
    def info_from_service(service_info: ServiceInfo) -> ZeroconfServiceInfo | None:
        """Return prepared info from mDNS entries."""
        properties = {}
        total_length = len(service_info.text)
        offset = 0
        while offset < total_length:
            (parsed_length,) = unpack_from("!B", service_info.text, offset)
            key_value = service_info.text[offset + 1 : offset + 1 + parsed_length].decode("UTF-8").split("=")
            properties[key_value[0]] = key_value[1]
            offset += parsed_length + 1

        return ZeroconfServiceInfo(
            address=service_info.addresses[0],
            hostname=service_info.server or "",
            port=service_info.port,
            properties=properties,
        )



"""
@generated by stubgen.  Do not edit manually!
isort:skip_file
"""
from __future__ import annotations
from .getnetworkoverview_pb2 import GetNetworkOverview
from devolo_plc_api.clients import Protobuf
from devolo_plc_api.zeroconf import ZeroconfServiceInfo as ZeroconfServiceInfo
from httpx import AsyncClient as AsyncClient

class PlcNetApi(Protobuf):
    password: str
    def __init__(self, ip: str, session: AsyncClient, info: ZeroconfServiceInfo) -> None: ...
    async def async_get_network_overview(self) -> GetNetworkOverview.LogicalNetwork: ...
    async def async_identify_device_start(self) -> bool: ...
    async def async_identify_device_stop(self) -> bool: ...
    async def async_pair_device(self) -> bool: ...
    async def async_set_user_device_name(self, name: str) -> bool: ...
    def get_network_overview(self) -> GetNetworkOverview.LogicalNetwork: ...
    def identify_device_start(self) -> bool: ...
    def identify_device_stop(self) -> bool: ...
    def pair_device(self) -> bool: ...
    def set_user_device_name(self, name: str) -> bool: ...



"""Test configuration."""
from __future__ import annotations

from collections import OrderedDict
from functools import partial
from typing import TYPE_CHECKING, AsyncGenerator, Generator
from unittest.mock import AsyncMock, Mock, patch

import pytest
import pytest_asyncio
from ifaddr import IP, Adapter

from devolo_plc_api import Device

from . import DeviceType, DifferentDirectoryExtension, TestData, load_test_data
from .mocks.zeroconf import MockAsyncServiceInfo, MockServiceBrowser

if TYPE_CHECKING:
    from syrupy.assertion import SnapshotAssertion

pytest_plugins = [
    "tests.fixtures.device_api",
    "tests.fixtures.plcnet_api",
]


@pytest.fixture(scope="session")
def test_data() -> TestData:
    """Load test data."""
    return load_test_data()


@pytest.fixture()
def block_communication() -> Generator[None, None, None]:
    """Block external communication."""
    adapter = OrderedDict()
    adapter["eth0"] = Adapter(name="eth0", nice_name="eth0", ips=[IP("192.0.2.100", network_prefix=24, nice_name="eth0")])
    with patch("devolo_plc_api.device.get_adapters", return_value=adapter.values()), patch(
        "devolo_plc_api.device.AsyncZeroconf", AsyncMock
    ), patch("devolo_plc_api.device.AsyncServiceInfo", MockAsyncServiceInfo):
        yield


@pytest_asyncio.fixture()
async def http_client() -> AsyncGenerator[None, None]:
    """Patch HTTP client."""
    with patch("devolo_plc_api.device.AsyncClient", autospec=True):
        yield


@pytest.fixture()
def mock_device(test_data: TestData) -> Device:
    """Generate a device from test data."""
    return Device(ip=test_data.ip)


@pytest.fixture()
def mock_info_from_service() -> Generator[Mock, None, None]:
    """Patch reading info from mDNS entries."""
    with patch("devolo_plc_api.device.Device.info_from_service") as ifs:
        yield ifs


@pytest.fixture(name="sleep")
def patch_sleep() -> Generator[AsyncMock, None, None]:
    """Don't sleep anywhere except if marked as allowed."""
    with patch("time.sleep"), patch("asyncio.sleep") as sleep:
        yield sleep


@pytest.fixture()
def service_browser(device_type: DeviceType) -> Generator[None, None, None]:
    """Patch mDNS service browser."""
    service_browser = partial(MockServiceBrowser, device_type=device_type)
    with patch("devolo_plc_api.device.AsyncServiceBrowser", service_browser), patch(
        "devolo_plc_api.network.ServiceBrowser", service_browser
    ):
        yield


@pytest.fixture()
def snapshot(snapshot: SnapshotAssertion) -> SnapshotAssertion:
    """Return snapshot assertion fixture with nicer path."""
    return snapshot.use_extension(DifferentDirectoryExtension)



"""Test communicating with a the plcnet API."""
from http import HTTPStatus

import pytest
from httpx import ConnectTimeout
from pytest_httpx import HTTPXMock

from devolo_plc_api import Device
from devolo_plc_api.exceptions import DevicePasswordProtected, DeviceUnavailable
from devolo_plc_api.plcnet_api import LogicalNetwork, PlcNetApi
from devolo_plc_api.plcnet_api.getnetworkoverview_pb2 import GetNetworkOverview
from devolo_plc_api.plcnet_api.identifydevice_pb2 import IdentifyDeviceResponse
from devolo_plc_api.plcnet_api.pairdevice_pb2 import PairDeviceStart
from devolo_plc_api.plcnet_api.setuserdevicename_pb2 import SetUserDeviceNameResponse

from . import DeviceType


class TestPlcApi:
    """Test devolo_plc_api.plcnet_api.plcnetapi.PlcNetApi class."""

    @pytest.mark.asyncio()
    @pytest.mark.parametrize("device_type", [DeviceType.PLC])
    @pytest.mark.usefixtures("block_communication", "service_browser")
    async def test_wrong_password_type(self, httpx_mock: HTTPXMock, mock_device: Device):
        """Test using different password hash if original password failed."""
        await mock_device.async_connect()
        assert mock_device.plcnet
        mock_device.password = "password"

        httpx_mock.add_response(status_code=HTTPStatus.UNAUTHORIZED)
        with pytest.raises(DevicePasswordProtected):
            await mock_device.plcnet.async_set_user_device_name("Test")
            assert mock_device.plcnet.password == "5e884898da28047151d0e56f8dc6292773603d0d6aabbdd62a11ef721d1542d8"

        httpx_mock.add_response(status_code=HTTPStatus.UNAUTHORIZED)
        with pytest.raises(DevicePasswordProtected):
            await mock_device.plcnet.async_set_user_device_name("Test")
            assert mock_device.plcnet.password == "113459eb7bb31bddee85ade5230d6ad5d8b2fb52879e00a84ff6ae1067a210d3"

        await mock_device.async_disconnect()

    @pytest.mark.asyncio()
    async def test_async_get_network_overview(self, plcnet_api: PlcNetApi, httpx_mock: HTTPXMock, network: LogicalNetwork):
        """Test getting the network overview asynchronously."""
        network_overview = GetNetworkOverview(network=network)
        httpx_mock.add_response(content=network_overview.SerializeToString())
        overview = await plcnet_api.async_get_network_overview()
        assert overview == network

    def test_get_network_overview(self, plcnet_api: PlcNetApi, httpx_mock: HTTPXMock, network: LogicalNetwork):
        """Test getting the network overview synchronously."""
        network_overview = GetNetworkOverview(network=network)
        httpx_mock.add_response(content=network_overview.SerializeToString())
        overview = plcnet_api.get_network_overview()
        assert overview == network

    @pytest.mark.asyncio()
    async def test_async_identify_device_start(self, plcnet_api: PlcNetApi, httpx_mock: HTTPXMock):
        """Test starting identifying a device asynchronously."""
        identify_device = IdentifyDeviceResponse()
        httpx_mock.add_response(content=identify_device.SerializeToString())
        assert await plcnet_api.async_identify_device_start()

    def test_identify_device_start(self, plcnet_api: PlcNetApi, httpx_mock: HTTPXMock):
        """Test starting identifying a device synchronously."""
        identify_device = IdentifyDeviceResponse()
        httpx_mock.add_response(content=identify_device.SerializeToString())
        assert plcnet_api.identify_device_start()

    @pytest.mark.asyncio()
    async def test_async_identify_device_stop(self, plcnet_api: PlcNetApi, httpx_mock: HTTPXMock):
        """Test stopping identifying a device asynchronously."""
        identify_device = IdentifyDeviceResponse()
        httpx_mock.add_response(content=identify_device.SerializeToString())
        assert await plcnet_api.async_identify_device_stop()

    def test_identify_device_stop(self, plcnet_api: PlcNetApi, httpx_mock: HTTPXMock):
        """Test stopping identifying a device synchronously."""
        identify_device = IdentifyDeviceResponse()
        httpx_mock.add_response(content=identify_device.SerializeToString())
        assert plcnet_api.identify_device_stop()

    @pytest.mark.asyncio()
    async def test_async_pair_device(self, plcnet_api: PlcNetApi, httpx_mock: HTTPXMock):
        """Test pairing a device asynchronously."""
        pair_device = PairDeviceStart()
        httpx_mock.add_response(content=pair_device.SerializeToString())
        assert await plcnet_api.async_pair_device()

    def test_pair_device(self, plcnet_api: PlcNetApi, httpx_mock: HTTPXMock):
        """Test pairing a device synchronously."""
        pair_device = PairDeviceStart()
        httpx_mock.add_response(content=pair_device.SerializeToString())
        assert plcnet_api.pair_device()

    @pytest.mark.asyncio()
    async def test_async_set_user_device_name(self, plcnet_api: PlcNetApi, httpx_mock: HTTPXMock):
        """Test setting a device name asynchronously."""
        user_device_name_set = SetUserDeviceNameResponse()
        httpx_mock.add_response(content=user_device_name_set.SerializeToString())
        assert await plcnet_api.async_set_user_device_name("Test")

    def test_set_user_device_name(self, plcnet_api: PlcNetApi, httpx_mock: HTTPXMock):
        """Test setting a device name synchronously."""
        user_device_name_set = SetUserDeviceNameResponse()
        httpx_mock.add_response(content=user_device_name_set.SerializeToString())
        assert plcnet_api.set_user_device_name("Test")

    @pytest.mark.asyncio()
    @pytest.mark.parametrize("device_type", [DeviceType.PLC])
    @pytest.mark.usefixtures("block_communication", "service_browser")
    async def test_device_unavailable(self, httpx_mock: HTTPXMock, mock_device: Device):
        """Test device being unavailable."""
        await mock_device.async_connect()
        assert mock_device.plcnet
        httpx_mock.add_exception(ConnectTimeout(""))
        with pytest.raises(DeviceUnavailable):
            await mock_device.plcnet.async_get_network_overview()

    @pytest.mark.asyncio()
    @pytest.mark.parametrize("device_type", [DeviceType.PLC])
    @pytest.mark.usefixtures("block_communication", "service_browser")
    async def test_attribute_error(self, mock_device: Device):
        """Test raising on calling not existing method."""
        await mock_device.async_connect()
        assert mock_device.plcnet
        with pytest.raises(AttributeError):
            mock_device.plcnet.test()  # type: ignore[union-attr]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: pairdevice.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10pairdevice.proto\x12\nplcnet.api\"&\n\x0fPairDeviceStart\x12\x13\n\x0bmac_address\x18\x01 \x01(\t\"\xba\x01\n\x12PairDeviceResponse\x12\x35\n\x06result\x18\x01 \x01(\x0e\x32%.plcnet.api.PairDeviceResponse.Result\"m\n\x06Result\x12\x0b\n\x07SUCCESS\x10\x00\x12\x13\n\x0fMACADDR_INVALID\x10\x01\x12\x13\n\x0fMACADDR_UNKNOWN\x10\x02\x12\x18\n\x13\x43OMMUNICATION_ERROR\x10\xfe\x01\x12\x12\n\rUNKNOWN_ERROR\x10\xff\x01\x42\x11\n\x06plcnetB\x07pairingb\x06proto3')



_PAIRDEVICESTART = DESCRIPTOR.message_types_by_name['PairDeviceStart']
_PAIRDEVICERESPONSE = DESCRIPTOR.message_types_by_name['PairDeviceResponse']
_PAIRDEVICERESPONSE_RESULT = _PAIRDEVICERESPONSE.enum_types_by_name['Result']
PairDeviceStart = _reflection.GeneratedProtocolMessageType('PairDeviceStart', (_message.Message,), {
  'DESCRIPTOR' : _PAIRDEVICESTART,
  '__module__' : 'pairdevice_pb2'
  # @@protoc_insertion_point(class_scope:plcnet.api.PairDeviceStart)
  })
_sym_db.RegisterMessage(PairDeviceStart)

PairDeviceResponse = _reflection.GeneratedProtocolMessageType('PairDeviceResponse', (_message.Message,), {
  'DESCRIPTOR' : _PAIRDEVICERESPONSE,
  '__module__' : 'pairdevice_pb2'
  # @@protoc_insertion_point(class_scope:plcnet.api.PairDeviceResponse)
  })
_sym_db.RegisterMessage(PairDeviceResponse)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\006plcnetB\007pairing'
  _PAIRDEVICESTART._serialized_start=32
  _PAIRDEVICESTART._serialized_end=70
  _PAIRDEVICERESPONSE._serialized_start=73
  _PAIRDEVICERESPONSE._serialized_end=259
  _PAIRDEVICERESPONSE_RESULT._serialized_start=150
  _PAIRDEVICERESPONSE_RESULT._serialized_end=259
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: getnetworkoverview.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18getnetworkoverview.proto\x12\nplcnet.api\"\xd5\x06\n\x12GetNetworkOverview\x12>\n\x07network\x18\x01 \x01(\x0b\x32-.plcnet.api.GetNetworkOverview.LogicalNetwork\x1a\x96\x04\n\x06\x44\x65vice\x12\x14\n\x0cproduct_name\x18\x01 \x01(\t\x12\x12\n\nproduct_id\x18\x02 \x01(\t\x12\x18\n\x10\x66riendly_version\x18\x03 \x01(\t\x12\x14\n\x0c\x66ull_version\x18\x04 \x01(\t\x12\x18\n\x10user_device_name\x18\x05 \x01(\t\x12\x19\n\x11user_network_name\x18\x06 \x01(\t\x12\x13\n\x0bmac_address\x18\x07 \x01(\t\x12@\n\x08topology\x18\x08 \x01(\x0e\x32..plcnet.api.GetNetworkOverview.Device.Topology\x12\x44\n\ntechnology\x18\t \x01(\x0e\x32\x30.plcnet.api.GetNetworkOverview.Device.Technology\x12\x17\n\x0f\x62ridged_devices\x18\n \x03(\t\x12\x14\n\x0cipv4_address\x18\x0b \x01(\t\x12\x1a\n\x12\x61ttached_to_router\x18\x0c \x01(\x08\"7\n\x08Topology\x12\x14\n\x10UNKNOWN_TOPOLOGY\x10\x00\x12\t\n\x05LOCAL\x10\x01\x12\n\n\x06REMOTE\x10\x02\"\\\n\nTechnology\x12\x16\n\x12UNKNOWN_TECHNOLOGY\x10\x00\x12\x14\n\x10HPAV_THUNDERBOLT\x10\x03\x12\x10\n\x0cHPAV_PANTHER\x10\x04\x12\x0e\n\nGHN_SPIRIT\x10\x07\x1a^\n\x08\x44\x61taRate\x12\x18\n\x10mac_address_from\x18\x01 \x01(\t\x12\x16\n\x0emac_address_to\x18\x02 \x01(\t\x12\x0f\n\x07tx_rate\x18\x03 \x01(\x01\x12\x0f\n\x07rx_rate\x18\x04 \x01(\x01\x1a\x85\x01\n\x0eLogicalNetwork\x12\x36\n\x07\x64\x65vices\x18\x01 \x03(\x0b\x32%.plcnet.api.GetNetworkOverview.Device\x12;\n\ndata_rates\x18\x02 \x03(\x0b\x32\'.plcnet.api.GetNetworkOverview.DataRateB\r\n\x06plcnetB\x03netb\x06proto3')



_GETNETWORKOVERVIEW = DESCRIPTOR.message_types_by_name['GetNetworkOverview']
_GETNETWORKOVERVIEW_DEVICE = _GETNETWORKOVERVIEW.nested_types_by_name['Device']
_GETNETWORKOVERVIEW_DATARATE = _GETNETWORKOVERVIEW.nested_types_by_name['DataRate']
_GETNETWORKOVERVIEW_LOGICALNETWORK = _GETNETWORKOVERVIEW.nested_types_by_name['LogicalNetwork']
_GETNETWORKOVERVIEW_DEVICE_TOPOLOGY = _GETNETWORKOVERVIEW_DEVICE.enum_types_by_name['Topology']
_GETNETWORKOVERVIEW_DEVICE_TECHNOLOGY = _GETNETWORKOVERVIEW_DEVICE.enum_types_by_name['Technology']
GetNetworkOverview = _reflection.GeneratedProtocolMessageType('GetNetworkOverview', (_message.Message,), {

  'Device' : _reflection.GeneratedProtocolMessageType('Device', (_message.Message,), {
    'DESCRIPTOR' : _GETNETWORKOVERVIEW_DEVICE,
    '__module__' : 'getnetworkoverview_pb2'
    # @@protoc_insertion_point(class_scope:plcnet.api.GetNetworkOverview.Device)
    })
  ,

  'DataRate' : _reflection.GeneratedProtocolMessageType('DataRate', (_message.Message,), {
    'DESCRIPTOR' : _GETNETWORKOVERVIEW_DATARATE,
    '__module__' : 'getnetworkoverview_pb2'
    # @@protoc_insertion_point(class_scope:plcnet.api.GetNetworkOverview.DataRate)
    })
  ,

  'LogicalNetwork' : _reflection.GeneratedProtocolMessageType('LogicalNetwork', (_message.Message,), {
    'DESCRIPTOR' : _GETNETWORKOVERVIEW_LOGICALNETWORK,
    '__module__' : 'getnetworkoverview_pb2'
    # @@protoc_insertion_point(class_scope:plcnet.api.GetNetworkOverview.LogicalNetwork)
    })
  ,
  'DESCRIPTOR' : _GETNETWORKOVERVIEW,
  '__module__' : 'getnetworkoverview_pb2'
  # @@protoc_insertion_point(class_scope:plcnet.api.GetNetworkOverview)
  })
_sym_db.RegisterMessage(GetNetworkOverview)
_sym_db.RegisterMessage(GetNetworkOverview.Device)
_sym_db.RegisterMessage(GetNetworkOverview.DataRate)
_sym_db.RegisterMessage(GetNetworkOverview.LogicalNetwork)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\006plcnetB\003net'
  _GETNETWORKOVERVIEW._serialized_start=41
  _GETNETWORKOVERVIEW._serialized_end=894
  _GETNETWORKOVERVIEW_DEVICE._serialized_start=128
  _GETNETWORKOVERVIEW_DEVICE._serialized_end=662
  _GETNETWORKOVERVIEW_DEVICE_TOPOLOGY._serialized_start=513
  _GETNETWORKOVERVIEW_DEVICE_TOPOLOGY._serialized_end=568
  _GETNETWORKOVERVIEW_DEVICE_TECHNOLOGY._serialized_start=570
  _GETNETWORKOVERVIEW_DEVICE_TECHNOLOGY._serialized_end=662
  _GETNETWORKOVERVIEW_DATARATE._serialized_start=664
  _GETNETWORKOVERVIEW_DATARATE._serialized_end=758
  _GETNETWORKOVERVIEW_LOGICALNETWORK._serialized_start=761
  _GETNETWORKOVERVIEW_LOGICALNETWORK._serialized_end=894
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class GetNetworkOverview(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class Device(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        class _Topology:
            ValueType = typing.NewType('ValueType', builtins.int)
            V: typing_extensions.TypeAlias = ValueType
        class _TopologyEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[GetNetworkOverview.Device._Topology.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            UNKNOWN_TOPOLOGY: GetNetworkOverview.Device._Topology.ValueType  # 0
            LOCAL: GetNetworkOverview.Device._Topology.ValueType  # 1
            """the device is LOCAL (i.e. reachable via eth/wifi)"""

            REMOTE: GetNetworkOverview.Device._Topology.ValueType  # 2
            """the device is REMOTE (i.e. connected via powerline to the LOCAL device)"""

        class Topology(_Topology, metaclass=_TopologyEnumTypeWrapper):
            pass

        UNKNOWN_TOPOLOGY: GetNetworkOverview.Device.Topology.ValueType  # 0
        LOCAL: GetNetworkOverview.Device.Topology.ValueType  # 1
        """the device is LOCAL (i.e. reachable via eth/wifi)"""

        REMOTE: GetNetworkOverview.Device.Topology.ValueType  # 2
        """the device is REMOTE (i.e. connected via powerline to the LOCAL device)"""


        class _Technology:
            ValueType = typing.NewType('ValueType', builtins.int)
            V: typing_extensions.TypeAlias = ValueType
        class _TechnologyEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[GetNetworkOverview.Device._Technology.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            UNKNOWN_TECHNOLOGY: GetNetworkOverview.Device._Technology.ValueType  # 0
            HPAV_THUNDERBOLT: GetNetworkOverview.Device._Technology.ValueType  # 3
            """HomePlugAV device (based on Intellon/QCA Thunderbolt or later, i.e. 6x00/74x0 family)"""

            HPAV_PANTHER: GetNetworkOverview.Device._Technology.ValueType  # 4
            """HomePlugAV device (based on Intellon/QCA Panther or later, i.e. 7420/75x0 family)"""

            GHN_SPIRIT: GetNetworkOverview.Device._Technology.ValueType  # 7
            """G.hn device (based on Maxlinear Spirit firmware)"""

        class Technology(_Technology, metaclass=_TechnologyEnumTypeWrapper):
            pass

        UNKNOWN_TECHNOLOGY: GetNetworkOverview.Device.Technology.ValueType  # 0
        HPAV_THUNDERBOLT: GetNetworkOverview.Device.Technology.ValueType  # 3
        """HomePlugAV device (based on Intellon/QCA Thunderbolt or later, i.e. 6x00/74x0 family)"""

        HPAV_PANTHER: GetNetworkOverview.Device.Technology.ValueType  # 4
        """HomePlugAV device (based on Intellon/QCA Panther or later, i.e. 7420/75x0 family)"""

        GHN_SPIRIT: GetNetworkOverview.Device.Technology.ValueType  # 7
        """G.hn device (based on Maxlinear Spirit firmware)"""


        PRODUCT_NAME_FIELD_NUMBER: builtins.int
        PRODUCT_ID_FIELD_NUMBER: builtins.int
        FRIENDLY_VERSION_FIELD_NUMBER: builtins.int
        FULL_VERSION_FIELD_NUMBER: builtins.int
        USER_DEVICE_NAME_FIELD_NUMBER: builtins.int
        USER_NETWORK_NAME_FIELD_NUMBER: builtins.int
        MAC_ADDRESS_FIELD_NUMBER: builtins.int
        TOPOLOGY_FIELD_NUMBER: builtins.int
        TECHNOLOGY_FIELD_NUMBER: builtins.int
        BRIDGED_DEVICES_FIELD_NUMBER: builtins.int
        IPV4_ADDRESS_FIELD_NUMBER: builtins.int
        ATTACHED_TO_ROUTER_FIELD_NUMBER: builtins.int
        product_name: typing.Text
        """product name for display purposes, like "devolo dLAN 1200+" """

        product_id: typing.Text
        """internal product identifier, like "MT2639" """

        friendly_version: typing.Text
        """version string for display purposes, like "2.5.0.0-1" """

        full_version: typing.Text
        """full version string"""

        user_device_name: typing.Text
        """user provided device name, if any"""

        user_network_name: typing.Text
        """user provided network name, if any"""

        mac_address: typing.Text
        """MAC address of the device, like "000B3BC3A4E6" """

        topology: global___GetNetworkOverview.Device.Topology.ValueType
        """in a real network, we will always see one LOCAL device
        and zero or more REMOTEs
        """

        technology: global___GetNetworkOverview.Device.Technology.ValueType
        """in a real network, you will always see either HPAV or GHN devices, but never both"""

        @property
        def bridged_devices(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]:
            """MAC addresses of devices attached to this device on the LAN side"""
            pass
        ipv4_address: typing.Text
        """ipv4 address of the device; empty if none is available"""

        attached_to_router: builtins.bool
        """indicates whether this adapter is attached to the router or not (only one should be set true)"""

        def __init__(self,
            *,
            product_name: typing.Text = ...,
            product_id: typing.Text = ...,
            friendly_version: typing.Text = ...,
            full_version: typing.Text = ...,
            user_device_name: typing.Text = ...,
            user_network_name: typing.Text = ...,
            mac_address: typing.Text = ...,
            topology: global___GetNetworkOverview.Device.Topology.ValueType = ...,
            technology: global___GetNetworkOverview.Device.Technology.ValueType = ...,
            bridged_devices: typing.Optional[typing.Iterable[typing.Text]] = ...,
            ipv4_address: typing.Text = ...,
            attached_to_router: builtins.bool = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["attached_to_router",b"attached_to_router","bridged_devices",b"bridged_devices","friendly_version",b"friendly_version","full_version",b"full_version","ipv4_address",b"ipv4_address","mac_address",b"mac_address","product_id",b"product_id","product_name",b"product_name","technology",b"technology","topology",b"topology","user_device_name",b"user_device_name","user_network_name",b"user_network_name"]) -> None: ...

    class DataRate(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        MAC_ADDRESS_FROM_FIELD_NUMBER: builtins.int
        MAC_ADDRESS_TO_FIELD_NUMBER: builtins.int
        TX_RATE_FIELD_NUMBER: builtins.int
        RX_RATE_FIELD_NUMBER: builtins.int
        mac_address_from: typing.Text
        """the powerline device from which we got information about the data rates"""

        mac_address_to: typing.Text
        """the powerline device to which we are transmitting/receiving"""

        tx_rate: builtins.float
        """transmit data rate in mbps"""

        rx_rate: builtins.float
        """receive data rate in mbps"""

        def __init__(self,
            *,
            mac_address_from: typing.Text = ...,
            mac_address_to: typing.Text = ...,
            tx_rate: builtins.float = ...,
            rx_rate: builtins.float = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["mac_address_from",b"mac_address_from","mac_address_to",b"mac_address_to","rx_rate",b"rx_rate","tx_rate",b"tx_rate"]) -> None: ...

    class LogicalNetwork(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        DEVICES_FIELD_NUMBER: builtins.int
        DATA_RATES_FIELD_NUMBER: builtins.int
        @property
        def devices(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___GetNetworkOverview.Device]:
            """all devices in the logical network"""
            pass
        @property
        def data_rates(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___GetNetworkOverview.DataRate]:
            """all known data rates between devices in the logical network"""
            pass
        def __init__(self,
            *,
            devices: typing.Optional[typing.Iterable[global___GetNetworkOverview.Device]] = ...,
            data_rates: typing.Optional[typing.Iterable[global___GetNetworkOverview.DataRate]] = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["data_rates",b"data_rates","devices",b"devices"]) -> None: ...

    NETWORK_FIELD_NUMBER: builtins.int
    @property
    def network(self) -> global___GetNetworkOverview.LogicalNetwork: ...
    def __init__(self,
        *,
        network: typing.Optional[global___GetNetworkOverview.LogicalNetwork] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["network",b"network"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["network",b"network"]) -> None: ...
global___GetNetworkOverview = GetNetworkOverview



"""Google Protobuf client."""
from __future__ import annotations

import asyncio
import logging
from abc import ABC, abstractmethod
from hashlib import sha256
from http import HTTPStatus
from typing import Any, Callable

from httpx import (
    AsyncClient,
    ConnectError,
    ConnectTimeout,
    DigestAuth,
    HTTPStatusError,
    ReadTimeout,
    RemoteProtocolError,
    Response,
)

from devolo_plc_api.exceptions import DevicePasswordProtected, DeviceUnavailable

TIMEOUT = 10.0


class Protobuf(ABC):
    """Google Protobuf client as ground work."""

    @abstractmethod
    def __init__(self) -> None:
        """Initialize the client."""
        self._logger = logging.getLogger(f"{self.__class__.__module__}.{self.__class__.__name__}")

        self.password: str

        self._ip: str
        self._path: str
        self._port: int
        self._session: AsyncClient
        self._user: str
        self._version: str

    def __getattr__(self, attr: str) -> Callable[..., Any]:
        """Catch attempts to call methods synchronously."""

        def method(*args: Any, **kwargs: Any) -> Any:
            return asyncio.run(getattr(self, async_method)(*args, **kwargs))

        async_method = f"async_{attr}"
        if hasattr(self.__class__, async_method):
            return method
        raise AttributeError(f"{self.__class__.__name__} object has no attribute {attr}")  # noqa: EM102, TRY003

    @property
    def url(self) -> str:
        """The base URL to query."""
        return f"http://{self._ip}:{self._port}/{self._path}/{self._version}/"

    async def _async_get(self, sub_url: str, timeout: float = TIMEOUT) -> Response:
        """Query URL asynchronously."""
        url = f"{self.url}{sub_url}"
        self._logger.debug("Getting from %s", url)
        return await self._async_request("GET", url, None, timeout)

    async def _async_post(self, sub_url: str, content: bytes, timeout: float = TIMEOUT) -> Response:
        """Post data asynchronously."""
        url = f"{self.url}{sub_url}"
        self._logger.debug("Posting to %s", url)
        return await self._async_request("POST", url, content, timeout)

    async def _async_request(self, method: str, url: str, content: bytes | None, timeout: float = TIMEOUT) -> Response:
        """Request data asynchronously."""
        try:
            response = await self._session.request(
                method,
                url,
                auth=DigestAuth(self._user, self.password),
                content=content,
                timeout=timeout,
            )
            if response.status_code == HTTPStatus.UNAUTHORIZED:
                self.password = sha256(self.password.encode("utf-8")).hexdigest()
                response = await self._session.request(
                    method,
                    url,
                    auth=DigestAuth(self._user, self.password),
                    content=content,
                    timeout=timeout,
                )
            response.raise_for_status()
        except HTTPStatusError as e:
            if e.response.status_code == HTTPStatus.UNAUTHORIZED:
                raise DevicePasswordProtected from None
            raise
        except (ConnectTimeout, ConnectError, ReadTimeout, RemoteProtocolError):
            raise DeviceUnavailable from None
        else:
            return response



"""Zeroconf dataclasses."""
from __future__ import annotations

from dataclasses import dataclass, field


@dataclass
class ZeroconfServiceInfo:
    """Prepared info from mDNS entries."""

    address: bytes = b""
    """IP address of the device."""

    port: int | None = None
    """mDNS port to use."""

    hostname: str = ""
    """mDNS hostname of the device."""

    properties: dict[str, str] = field(default_factory=dict)
    """Properties provided by the device."""



"""Implementation of the devolo plcnet API."""
from __future__ import annotations

from typing import TYPE_CHECKING

from devolo_plc_api.clients import Protobuf

from .getnetworkoverview_pb2 import GetNetworkOverview
from .identifydevice_pb2 import IdentifyDeviceResponse, IdentifyDeviceStart, IdentifyDeviceStop
from .pairdevice_pb2 import PairDeviceResponse, PairDeviceStart
from .setuserdevicename_pb2 import SetUserDeviceName, SetUserDeviceNameResponse

if TYPE_CHECKING:
    from httpx import AsyncClient

    from devolo_plc_api.zeroconf import ZeroconfServiceInfo


class PlcNetApi(Protobuf):
    """
    Implementation of the devolo plcnet API.

    :param ip: IP address of the device to communicate with
    :param session: HTTP client session
    :param info: Information collected from the mDNS query
    """

    def __init__(self, ip: str, session: AsyncClient, info: ZeroconfServiceInfo) -> None:
        """Initialize the plcnet API."""
        super().__init__()

        self._ip = ip
        self._mac = info.properties["PlcMacAddress"]
        self._path = info.properties["Path"]
        self._port = info.port
        self._session = session
        self._user = "devolo"
        self._version = info.properties["Version"]

        self.password = ""

    async def async_get_network_overview(self) -> GetNetworkOverview.LogicalNetwork:
        """
        Get a PLC network overview.

        :return: Network overview
        """
        self._logger.debug("Getting network overview.")
        network_overview = GetNetworkOverview()
        response = await self._async_get("GetNetworkOverview")
        network_overview.ParseFromString(await response.aread())
        return network_overview.network

    async def async_identify_device_start(self) -> bool:
        """
        Make PLC LED of a device blink to identify it.

        :return: True, if identifying was successfully started, otherwise False
        """
        self._logger.debug("Starting LED blinking.")
        identify_device = IdentifyDeviceStart()
        identify_device.mac_address = self._mac
        query = await self._async_post("IdentifyDeviceStart", content=identify_device.SerializeToString())
        response = IdentifyDeviceResponse()
        response.ParseFromString(await query.aread())
        return response.result == response.SUCCESS

    async def async_identify_device_stop(self) -> bool:
        """
        Stop the PLC LED blinking.

        :return: True, if identifying was successfully stopped, otherwise False
        """
        self._logger.debug("Stopping LED blinking.")
        identify_device = IdentifyDeviceStop()
        identify_device.mac_address = self._mac
        query = await self._async_post("IdentifyDeviceStop", content=identify_device.SerializeToString())
        response = IdentifyDeviceResponse()
        response.ParseFromString(await query.aread())
        return response.result == response.SUCCESS

    async def async_pair_device(self) -> bool:
        """
        Start pairing mode.

        :return: True, if pairing was started successfully, otherwise False
        """
        self._logger.debug("Pairing.")
        pair_device = PairDeviceStart()
        pair_device.mac_address = self._mac
        query = await self._async_post("PairDeviceStart", content=pair_device.SerializeToString())
        response = PairDeviceResponse()
        response.ParseFromString(await query.aread())
        return response.result == response.SUCCESS

    async def async_set_user_device_name(self, name: str) -> bool:
        """
        Set device name.

        :param name: Name, the device shall have
        :return: True, if the device was successfully renamed, otherwise False
        """
        self._logger.debug("Setting device name.")
        set_user_name = SetUserDeviceName()
        set_user_name.mac_address = self._mac
        set_user_name.user_device_name = name
        query = await self._async_post("SetUserDeviceName", content=set_user_name.SerializeToString())
        response = SetUserDeviceNameResponse()
        response.ParseFromString(await query.aread())
        return response.result == response.SUCCESS



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class WifiMultiApGetResponse(google.protobuf.message.Message):
    """Details about MultiAP as returned by the 'WifiMultiApGet' endpoint."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ENABLED_FIELD_NUMBER: builtins.int
    CONTROLLER_ID_FIELD_NUMBER: builtins.int
    CONTROLLER_IP_FIELD_NUMBER: builtins.int
    enabled: builtins.bool
    """Describes if the MultiAP functionality is enabled in the device."""
    controller_id: builtins.str
    """The id of the mesh controller, in form of its MAC address,
    if a mesh controller is known to the device.
    If the device is not aware of a mesh controller, e.g. because
    none has been elected yet, it is left empty.

    The MAC address is represented as a string of 12 hexadecimal
    digits (digits 0-9, letters A-F or a-f) displayed as six pairs of
    digits separated by colons.
    """
    controller_ip: builtins.str
    """The IP address of the known mesh controller, if the implementation
    provides it.
    If the device is not aware of a mesh controller or doesn't
    know its IP, it is left empty.

    The IP can be an IPv4 in dot-separated decimal format, or an IPv6
    in colon-separated hexadecimal format. In case multiple IPs are
    known, the value can be a comma-separated string of either formats.
    Also, an IP can optionally be prefixed with an identifier separated
    from the IP with a semicolon.
    """
    def __init__(
        self,
        *,
        enabled: builtins.bool = ...,
        controller_id: builtins.str = ...,
        controller_ip: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["controller_id", b"controller_id", "controller_ip", b"controller_ip", "enabled", b"enabled"]) -> None: ...

global___WifiMultiApGetResponse = WifiMultiApGetResponse



"""Helper methods to allow advanced usage of information provided by the device."""
from io import BytesIO
from typing import Any

from segno.helpers import make_wifi

from devolo_plc_api.device_api import WifiGuestAccessGet
from devolo_plc_api.device_api.wifinetwork_pb2 import WPA_NONE


def wifi_qr_code(guest_wifi: WifiGuestAccessGet, kind: str = "svg", **kwargs: Any) -> bytes:
    """
    Generate a wifi QR code.

    :param guest_wifi: Wifi credentials to represent in the QR code
    :param kind: Output format of the image
    :return: Bytes of image
    """
    buffer = BytesIO()
    qr_code = make_wifi(ssid=guest_wifi.ssid, password=guest_wifi.key, security=None if guest_wifi.wpa == WPA_NONE else "WPA")
    qr_code.save(out=buffer, kind=kind, **kwargs)
    return buffer.getvalue()



"""The devolo device API."""
import re

from .deviceapi import DeviceApi
from .multiap_pb2 import WifiMultiApGetResponse
from .support_pb2 import SupportInfoDump
from .updatefirmware_pb2 import UpdateFirmwareCheck
from .wifinetwork_pb2 import (
    WIFI_BAND_2G,
    WIFI_BAND_5G,
    WIFI_VAP_GUEST_AP,
    WIFI_VAP_MAIN_AP,
    WIFI_VAP_STATION,
    WifiConnectedStationsGet,
    WifiGuestAccessGet,
    WifiNeighborAPsGet,
    WifiRepeatedAPsGet,
)

CONFIGLAYER_FORMAT = re.compile(rb"(([A-Z][A-Z0-9._]+)=(.+?))(?=(([A-Z][A-Z0-9._]+)=(.+))|$)", re.DOTALL)
SERVICE_TYPE = "_dvl-deviceapi._tcp.local."
UPDATE_AVAILABLE = UpdateFirmwareCheck.UPDATE_AVAILABLE
UPDATE_NOT_AVAILABLE = UpdateFirmwareCheck.UPDATE_NOT_AVAILABLE

RepeatedAPInfo = WifiRepeatedAPsGet.RepeatedAPInfo
ConnectedStationInfo = WifiConnectedStationsGet.ConnectedStationInfo
NeighborAPInfo = WifiNeighborAPsGet.NeighborAPInfo
SupportInfoItem = SupportInfoDump.SupportInfoItem

__all__ = [
    "ConnectedStationInfo",
    "DeviceApi",
    "NeighborAPInfo",
    "RepeatedAPInfo",
    "SupportInfoItem",
    "WifiGuestAccessGet",
    "WifiMultiApGetResponse",
    "CONFIGLAYER_FORMAT",
    "SERVICE_TYPE",
    "UPDATE_AVAILABLE",
    "UPDATE_NOT_AVAILABLE",
    "WIFI_BAND_2G",
    "WIFI_BAND_5G",
    "WIFI_VAP_GUEST_AP",
    "WIFI_VAP_MAIN_AP",
    "WIFI_VAP_STATION",
]



"""Exceptions that can occur when determining features."""


class FeatureNotSupported(Exception):
    """The feature is not supported by your device."""

    def __init__(self, feature: str) -> None:
        """Initialize error."""
        super().__init__(f"The device does not support {feature}.")



"""Fixtures for device API tests."""
from __future__ import annotations

from secrets import randbelow
from typing import TYPE_CHECKING, AsyncGenerator

import pytest
import pytest_asyncio
from httpx import AsyncClient

from devolo_plc_api.device_api import (
    SERVICE_TYPE,
    ConnectedStationInfo,
    DeviceApi,
    NeighborAPInfo,
    RepeatedAPInfo,
    SupportInfoItem,
    UpdateFirmwareCheck,
)

if TYPE_CHECKING:
    from tests import TestData


@pytest_asyncio.fixture()
async def device_api(test_data: TestData, feature: str) -> AsyncGenerator[DeviceApi, None]:
    """Yield a prepared DeviceApi object."""
    test_data.device_info[SERVICE_TYPE].properties["Features"] = feature
    async with AsyncClient() as client:
        yield DeviceApi(test_data.ip, client, test_data.device_info[SERVICE_TYPE])


@pytest.fixture(scope="session")
def connected_station() -> ConnectedStationInfo:
    """Generate a mocked answer of a connected wifi station."""
    station = ConnectedStationInfo()
    station.mac_address = "aa:bb:cc:dd:ee:ff"
    return station


@pytest.fixture(scope="session")
def firmware_update() -> UpdateFirmwareCheck:
    """Generate a mocked firmware update message."""
    update = UpdateFirmwareCheck()
    update.result = UpdateFirmwareCheck.UPDATE_NOT_AVAILABLE
    update.new_firmware_version = ""
    return update


@pytest.fixture(scope="session")
def neighbor_ap() -> NeighborAPInfo:
    """Generate a mocked answer of a neighbor access point."""
    ap = NeighborAPInfo()
    ap.mac_address = "aa:bb:cc:dd:ee:ff"
    return ap


@pytest.fixture(scope="session")
def repeated_ap() -> RepeatedAPInfo:
    """Generate a mocked answer of a repeated access point."""
    ap = RepeatedAPInfo()
    ap.mac_address = "aa:bb:cc:dd:ee:ff"
    return ap


@pytest.fixture(scope="session")
def runtime() -> int:
    """Generate a mocked runtime of a device."""
    return randbelow(65536)


@pytest.fixture(scope="session")
def support_item() -> SupportInfoItem:
    """Generate mocked support information."""
    return SupportInfoItem(label="test", content=b"test")



"""Exceptions that can occur when communicating with a devolo device."""


class DeviceNotFound(Exception):
    """The device was not found."""

    def __init__(self, ip: str) -> None:
        """Initialize error."""
        super().__init__(f"The device {ip} did not answer.")


class DeviceUnavailable(Exception):
    """The device is not available, e.g. in standby."""

    def __init__(self) -> None:
        """Initialize error."""
        super().__init__("The device is currently not available. Maybe on standby?")


class DevicePasswordProtected(Exception):
    """The device is password protected."""

    def __init__(self) -> None:
        """Initialize error."""
        super().__init__("The used password is wrong.")



"""Mock methods from the Zeroconf module."""
from __future__ import annotations

import socket
from typing import Any, Callable
from unittest.mock import AsyncMock, Mock

from zeroconf import ServiceStateChange, Zeroconf
from zeroconf.asyncio import AsyncServiceInfo

from devolo_plc_api.plcnet_api import SERVICE_TYPE

from tests import DeviceType, load_test_data


class MockServiceBrowser:
    """Mock of the ServiceBrowser."""

    _async_start = AsyncMock()
    _async_cancel = AsyncMock()
    async_cancel = AsyncMock()
    cancel = Mock()

    def __init__(self, zeroconf: Zeroconf, type_: list[str] | str, handlers: list[Callable], **kwargs: Any) -> None:
        """Initialize the service browser."""
        if isinstance(type_, str):
            type_ = [type_]
        elif kwargs.get("device_type") == DeviceType.REPEATER:
            type_.remove(SERVICE_TYPE)
        for service_type in type_:
            handlers[0](zeroconf, service_type, service_type, ServiceStateChange.Added)


class MockAsyncServiceInfo(AsyncServiceInfo):
    """AsyncServiceInfo object with pre-filled information."""

    async_request = AsyncMock()

    def __init__(self, service_type: str, name: str) -> None:
        """Initialize the service info."""
        test_data = load_test_data()
        super().__init__(
            service_type,
            name,
            port=test_data.device_info[service_type].port,
            properties=test_data.device_info[service_type].properties,
            server=test_data.hostname,
            addresses=[socket.inet_aton(test_data.ip)],
        )



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class FactoryResetStart(google.protobuf.message.Message):
    """
    Perform a factory reset on delos devices
    Use GET, no payload required
    The result shows only the success of the triggering
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class _Result:
        ValueType = typing.NewType('ValueType', builtins.int)
        V: typing_extensions.TypeAlias = ValueType
    class _ResultEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[FactoryResetStart._Result.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        SUCCESS: FactoryResetStart._Result.ValueType  # 0
        UNKNOWN_ERROR: FactoryResetStart._Result.ValueType  # 255
    class Result(_Result, metaclass=_ResultEnumTypeWrapper):
        pass

    SUCCESS: FactoryResetStart.Result.ValueType  # 0
    UNKNOWN_ERROR: FactoryResetStart.Result.ValueType  # 255

    RESULT_FIELD_NUMBER: builtins.int
    result: global___FactoryResetStart.Result.ValueType
    """SUCCESS, if factory reset was triggered correct"""

    def __init__(self,
        *,
        result: global___FactoryResetStart.Result.ValueType = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["result",b"result"]) -> None: ...
global___FactoryResetStart = FactoryResetStart



"""The devolo plcnet API."""
from .getnetworkoverview_pb2 import GetNetworkOverview
from .plcnetapi import PlcNetApi

DEVICES_WITHOUT_PLCNET = ["3046", "3047", "3048", "3049", "3254", "3255", "3256"]
GHN_SPIRIT = GetNetworkOverview.Device.GHN_SPIRIT
HPAV_PANTHER = GetNetworkOverview.Device.HPAV_PANTHER
HPAV_THUNDERBOLT = GetNetworkOverview.Device.HPAV_THUNDERBOLT
LOCAL = GetNetworkOverview.Device.LOCAL
REMOTE = GetNetworkOverview.Device.REMOTE
SERVICE_TYPE = "_dvl-plcnetapi._tcp.local."

DataRate = GetNetworkOverview.DataRate
Device = GetNetworkOverview.Device
LogicalNetwork = GetNetworkOverview.LogicalNetwork

__all__ = [
    "DataRate",
    "Device",
    "LogicalNetwork",
    "PlcNetApi",
    "DEVICES_WITHOUT_PLCNET",
    "GHN_SPIRIT",
    "HPAV_PANTHER",
    "HPAV_THUNDERBOLT",
    "LOCAL",
    "REMOTE",
    "SERVICE_TYPE",
]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: support.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rsupport.proto\x12\ndevice.api\"\x80\x01\n\x0fSupportInfoDump\x12:\n\x05items\x18\x01 \x03(\x0b\x32+.device.api.SupportInfoDump.SupportInfoItem\x1a\x31\n\x0fSupportInfoItem\x12\r\n\x05label\x18\x01 \x01(\t\x12\x0f\n\x07\x63ontent\x18\x02 \x01(\x0c\"\xb1\x01\n\x17SupportInfoDumpResponse\x12:\n\x06result\x18\x01 \x01(\x0e\x32*.device.api.SupportInfoDumpResponse.Result\x12)\n\x04info\x18\x03 \x01(\x0b\x32\x1b.device.api.SupportInfoDump\")\n\x06Result\x12\x0b\n\x07SUCCESS\x10\x00\x12\x12\n\rUNKNOWN_ERROR\x10\xff\x01J\x04\x08\x02\x10\x03\x42\x11\n\x06\x64\x65viceB\x07supportb\x06proto3')



_SUPPORTINFODUMP = DESCRIPTOR.message_types_by_name['SupportInfoDump']
_SUPPORTINFODUMP_SUPPORTINFOITEM = _SUPPORTINFODUMP.nested_types_by_name['SupportInfoItem']
_SUPPORTINFODUMPRESPONSE = DESCRIPTOR.message_types_by_name['SupportInfoDumpResponse']
_SUPPORTINFODUMPRESPONSE_RESULT = _SUPPORTINFODUMPRESPONSE.enum_types_by_name['Result']
SupportInfoDump = _reflection.GeneratedProtocolMessageType('SupportInfoDump', (_message.Message,), {

  'SupportInfoItem' : _reflection.GeneratedProtocolMessageType('SupportInfoItem', (_message.Message,), {
    'DESCRIPTOR' : _SUPPORTINFODUMP_SUPPORTINFOITEM,
    '__module__' : 'support_pb2'
    # @@protoc_insertion_point(class_scope:device.api.SupportInfoDump.SupportInfoItem)
    })
  ,
  'DESCRIPTOR' : _SUPPORTINFODUMP,
  '__module__' : 'support_pb2'
  # @@protoc_insertion_point(class_scope:device.api.SupportInfoDump)
  })
_sym_db.RegisterMessage(SupportInfoDump)
_sym_db.RegisterMessage(SupportInfoDump.SupportInfoItem)

SupportInfoDumpResponse = _reflection.GeneratedProtocolMessageType('SupportInfoDumpResponse', (_message.Message,), {
  'DESCRIPTOR' : _SUPPORTINFODUMPRESPONSE,
  '__module__' : 'support_pb2'
  # @@protoc_insertion_point(class_scope:device.api.SupportInfoDumpResponse)
  })
_sym_db.RegisterMessage(SupportInfoDumpResponse)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\006deviceB\007support'
  _SUPPORTINFODUMP._serialized_start=30
  _SUPPORTINFODUMP._serialized_end=158
  _SUPPORTINFODUMP_SUPPORTINFOITEM._serialized_start=109
  _SUPPORTINFODUMP_SUPPORTINFOITEM._serialized_end=158
  _SUPPORTINFODUMPRESPONSE._serialized_start=161
  _SUPPORTINFODUMPRESPONSE._serialized_end=338
  _SUPPORTINFODUMPRESPONSE_RESULT._serialized_start=291
  _SUPPORTINFODUMPRESPONSE_RESULT._serialized_end=332
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: setuserdevicename.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17setuserdevicename.proto\x12\nplcnet.api\"B\n\x11SetUserDeviceName\x12\x13\n\x0bmac_address\x18\x01 \x01(\t\x12\x18\n\x10user_device_name\x18\x02 \x01(\t\"\xe2\x01\n\x19SetUserDeviceNameResponse\x12<\n\x06result\x18\x01 \x01(\x0e\x32,.plcnet.api.SetUserDeviceNameResponse.Result\"\x86\x01\n\x06Result\x12\x0b\n\x07SUCCESS\x10\x00\x12\x13\n\x0fMACADDR_INVALID\x10\x01\x12\x13\n\x0fMACADDR_UNKNOWN\x10\x02\x12\x17\n\x13\x44\x45VICE_NAME_INVALID\x10\x03\x12\x18\n\x13\x43OMMUNICATION_ERROR\x10\xfe\x01\x12\x12\n\rUNKNOWN_ERROR\x10\xff\x01\x42\x0e\n\x06plcnetB\x04nameb\x06proto3')



_SETUSERDEVICENAME = DESCRIPTOR.message_types_by_name['SetUserDeviceName']
_SETUSERDEVICENAMERESPONSE = DESCRIPTOR.message_types_by_name['SetUserDeviceNameResponse']
_SETUSERDEVICENAMERESPONSE_RESULT = _SETUSERDEVICENAMERESPONSE.enum_types_by_name['Result']
SetUserDeviceName = _reflection.GeneratedProtocolMessageType('SetUserDeviceName', (_message.Message,), {
  'DESCRIPTOR' : _SETUSERDEVICENAME,
  '__module__' : 'setuserdevicename_pb2'
  # @@protoc_insertion_point(class_scope:plcnet.api.SetUserDeviceName)
  })
_sym_db.RegisterMessage(SetUserDeviceName)

SetUserDeviceNameResponse = _reflection.GeneratedProtocolMessageType('SetUserDeviceNameResponse', (_message.Message,), {
  'DESCRIPTOR' : _SETUSERDEVICENAMERESPONSE,
  '__module__' : 'setuserdevicename_pb2'
  # @@protoc_insertion_point(class_scope:plcnet.api.SetUserDeviceNameResponse)
  })
_sym_db.RegisterMessage(SetUserDeviceNameResponse)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\006plcnetB\004name'
  _SETUSERDEVICENAME._serialized_start=39
  _SETUSERDEVICENAME._serialized_end=105
  _SETUSERDEVICENAMERESPONSE._serialized_start=108
  _SETUSERDEVICENAMERESPONSE._serialized_end=334
  _SETUSERDEVICENAMERESPONSE_RESULT._serialized_start=200
  _SETUSERDEVICENAMERESPONSE_RESULT._serialized_end=334
# @@protoc_insertion_point(module_scope)



"""
@generated by stubgen.  Do not edit manually!
isort:skip_file
"""
from __future__ import annotations
from .multiap_pb2 import WifiMultiApGetResponse
from .support_pb2 import SupportInfoDump
from .updatefirmware_pb2 import UpdateFirmwareCheck
from .wifinetwork_pb2 import WifiConnectedStationsGet, WifiGuestAccessGet, WifiNeighborAPsGet, WifiRepeatedAPsGet
from devolo_plc_api.clients import Protobuf
from devolo_plc_api.zeroconf import ZeroconfServiceInfo as ZeroconfServiceInfo
from httpx import AsyncClient as AsyncClient

class DeviceApi(Protobuf):
    features: list[str]
    password: str
    def __init__(self, ip: str, session: AsyncClient, info: ZeroconfServiceInfo) -> None: ...
    async def async_get_led_setting(self) -> bool: ...
    async def async_set_led_setting(self, enable: bool) -> bool: ...
    async def async_get_wifi_multi_ap(self) -> WifiMultiApGetResponse: ...
    async def async_get_wifi_repeated_access_points(self) -> list[WifiRepeatedAPsGet.RepeatedAPInfo]: ...
    async def async_start_wps_clone(self) -> bool: ...
    async def async_factory_reset(self) -> bool: ...
    async def async_restart(self) -> bool: ...
    async def async_uptime(self) -> int: ...
    async def async_get_support_info(self) -> SupportInfoDump: ...
    async def async_check_firmware_available(self) -> UpdateFirmwareCheck: ...
    async def async_start_firmware_update(self) -> bool: ...
    async def async_get_wifi_connected_station(self) -> list[WifiConnectedStationsGet.ConnectedStationInfo]: ...
    async def async_get_wifi_guest_access(self) -> WifiGuestAccessGet: ...
    async def async_set_wifi_guest_access(self, enable: bool, duration: int = ...) -> bool: ...
    async def async_get_wifi_neighbor_access_points(self) -> list[WifiNeighborAPsGet.NeighborAPInfo]: ...
    async def async_start_wps(self) -> bool: ...
    def get_led_setting(self) -> bool: ...
    def set_led_setting(self, enable: bool) -> bool: ...
    def get_wifi_multi_ap(self) -> WifiMultiApGetResponse: ...
    def get_wifi_repeated_access_points(self) -> list[WifiRepeatedAPsGet.RepeatedAPInfo]: ...
    def start_wps_clone(self) -> bool: ...
    def factory_reset(self) -> bool: ...
    def restart(self) -> bool: ...
    def uptime(self) -> int: ...
    def get_support_info(self) -> SupportInfoDump: ...
    def check_firmware_available(self) -> UpdateFirmwareCheck: ...
    def start_firmware_update(self) -> bool: ...
    def get_wifi_connected_station(self) -> list[WifiConnectedStationsGet.ConnectedStationInfo]: ...
    def get_wifi_guest_access(self) -> WifiGuestAccessGet: ...
    def set_wifi_guest_access(self, enable: bool, duration: int = ...) -> bool: ...
    def get_wifi_neighbor_access_points(self) -> list[WifiNeighborAPsGet.NeighborAPInfo]: ...
    def start_wps(self) -> bool: ...



import asyncio

from devolo_plc_api import Device, wifi_qr_code

# IP of the device to query
IP = "192.168.0.10"

# Password, if the device has one. It is the same as the Web-UI has. It no password is set, you can remove the password
# parameter or set it to None.
PASSWORD = "super_secret"


async def run():
    async with Device(ip=IP) as dpa:
        # Set the password
        dpa.password = PASSWORD

        # Get LED settings of the device. The state might be LED_ON or LED_OFF.
        print("LED is on" if await dpa.device.async_get_led_setting() else "LED if off")

        # Set LED settings of the device. Set enable to True to them turn on, to False to turn them off.
        # If the state was changed successfully, True is returned, otherwise False.
        print("success" if await dpa.device.async_set_led_setting(enable=True) else "failed")

        # Get MultiAP details. If the device is not aware of a mesh controller or doesn't know its IP, it is left empty.
        multi_ap = await dpa.device.async_get_wifi_multi_ap()
        print(multi_ap.enabled)  # True
        print(multi_ap.controller_id)  # "AA:BB:CC:DD:EE:FF"
        print(multi_ap.controller_ip)  # "192.0.2.1"

        # Factory reset the device. If the reset will happen shortly, True is returned, otherwise False.
        print("success" if await dpa.device.async_factory_reset() else "failed")

        # Restart the device. If the restart will happen shortly, True is returned, otherwise False.
        print("success" if await dpa.device.async_restart() else "failed")

        # Get uptime of the device. This value can only be used as a strict monotonically increasing number and therefore has no unit.
        print(await dpa.device.async_uptime())

        # Get support information from the device.
        print(await dpa.device.async_get_support_info())

        # Check for new firmware versions
        firmware = await dpa.device.async_check_firmware_available()
        print(firmware.result)  # devolo_plc_api.device_api.UPDATE_NOT_AVAILABLE
        print(firmware.new_firmware_version)  # ""

        # Start firmware update, if new version is available. Important: The response does not tell you anything about the
        # success of the update itself.
        print("update started" if await dpa.device.async_start_firmware_update() else "no update available")

        # Get details of wifi stations connected to the device: MAC address, access point type (main or guest), wifi band and
        # connection rates.
        connected_stations = await dpa.device.async_get_wifi_connected_station()
        print(connected_stations[0].mac_address)  # "AA:BB:CC:DD:EE:FF"
        print(connected_stations[0].vap_type)  # devolo_plc_api.device_api.WIFI_VAP_MAIN_AP
        print(connected_stations[0].band)  # devolo_plc_api.device_api.WIFI_BAND_5G
        print(connected_stations[0].rx_rate)  # 87800
        print(connected_stations[0].tx_rate)  # 87800

        # Get details about wifi guest access: SSID, Wifi key, state (enabled/disabled) and if time limited, the remaining
        # duration.
        guest_wifi = await dpa.device.async_get_wifi_guest_access()
        print(guest_wifi.ssid)  # "devolo-guest-930"
        print(guest_wifi.key)  # "HMANPGBA"
        print(guest_wifi.enabled)  # False
        print(guest_wifi.remaining_duration)  # 0

        # Get a QR code of the guest wifi settings as byte stream in SVG format
        qr = wifi_qr_code(guest_wifi)
        with open("qr.svg", "wb") as binary_file:
            binary_file.write(qr)

        # Enable or disable the wifi guest access. Set enable to True to it turn on, to False to turn it off. Optionally
        # specify a duration in minutes. Changing SSID or the wifi key is currently not supported. If the state was changed
        # successfully, True is returned, otherwise False.
        print("success" if await dpa.device.async_set_wifi_guest_access(enable=True, duration=5) else "failed")

        # Get details about other access points in your neighborhood: MAC address, SSID, wifi band, used channel, signal
        # strength in DB and a value from 1 to 5, if you would want to map the signal strength to a signal bars.
        neighbor_aps = await dpa.device.async_get_wifi_neighbor_access_points()
        print(neighbor_aps[0].mac_address)  # "AA:BB:CC:DD:EE:FF"
        print(neighbor_aps[0].ssid)  # "wifi"
        print(neighbor_aps[0].band)  # devolo_plc_api.device_api.WIFI_BAND_2G
        print(neighbor_aps[0].channel)  # 1
        print(neighbor_aps[0].signal)  # -73
        print(neighbor_aps[0].signal_bars)  # 1

        # Start WPS push button configuration. If WPS was started successfully, True is returned, otherwise False.
        print("WPS started" if await dpa.device.async_start_wps() else "WPS start failed")

        # Start WPS clone mode. If clone mode was started successfully, True is returned, otherwise False.
        print("WPS clone mode started" if await dpa.device.async_start_wps_clone() else "WPS clone mode start failed")

        # Get PLC network overview with enriched information like firmware version.
        network = await dpa.plcnet.async_get_network_overview()
        print(network.devices[0].product_name)  # "devolo Magic 2 WiFi next"
        print(network.devices[0].product_id)  # "MT3056"
        print(network.devices[0].friendly_version)  # "7.12.5.124"
        print(network.devices[0].full_version)  # "magic-2-wifi-next 7.12.5.124_2022-08-29"
        print(network.devices[0].user_device_name)  # "Living Room"
        print(network.devices[0].mac_address)  # "AABBCCDDEEFF"
        print(network.devices[0].topology)  # devolo_plc_api.plcnet_api.LOCAL
        print(network.devices[0].technology)  # devolo_plc_api.plcnet_api.GHN_SPIRIT
        print(network.devices[0].bridged_devices)  # []
        print(network.devices[0].attached_to_router)  # True
        print(network.devices[0].user_network_name)  # ""
        print(network.devices[0].ipv4_address)  # ""
        print(network.data_rates[0].mac_address_from)  # "AABBCCDDEEFF"
        print(network.data_rates[0].mac_address_to)  # "AABBCCDDEEFF"
        print(network.data_rates[0].tx_rate)  # 129.9375
        print(network.data_rates[0].rx_rate)  # 124.6875

        # Identify the device by making the PLC LED blink. This call returns directly with True, if identifying was started
        # successfully, otherwise False. However, the LED stays blinking for two minutes.
        print("success" if await dpa.plcnet.async_identify_device_start() else "failed")

        # Stop identify the device if you don't want to wait for the timeout.
        print("success" if await dpa.plcnet.async_identify_device_stop() else "failed")

        # Start pairing the device. This call returns directly with True, if pairing was started successfully, otherwise
        # False. However, the device stays in pairing mode for up to three minutes.
        print("success" if await dpa.plcnet.async_pair_device() else "failed")

        # Set the user device name. If the name was changed successfully, True is returned, otherwise False.
        print("success" if await dpa.plcnet.async_set_user_device_name(name="New name") else "failed")


if __name__ == "__main__":
    asyncio.run(run())



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: restart.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rrestart.proto\x12\ndevice.api\"\x80\x01\n\x0fRestartResponse\x12\x32\n\x06result\x18\x01 \x01(\x0e\x32\".device.api.RestartResponse.Result\x12\x0e\n\x06uptime\x18\x02 \x01(\x03\")\n\x06Result\x12\x0b\n\x07SUCCESS\x10\x00\x12\x12\n\rUNKNOWN_ERROR\x10\xff\x01\"#\n\x11UptimeGetResponse\x12\x0e\n\x06uptime\x18\x02 \x01(\x03\x42\x11\n\x06\x64\x65viceB\x07restartb\x06proto3')



_RESTARTRESPONSE = DESCRIPTOR.message_types_by_name['RestartResponse']
_UPTIMEGETRESPONSE = DESCRIPTOR.message_types_by_name['UptimeGetResponse']
_RESTARTRESPONSE_RESULT = _RESTARTRESPONSE.enum_types_by_name['Result']
RestartResponse = _reflection.GeneratedProtocolMessageType('RestartResponse', (_message.Message,), {
  'DESCRIPTOR' : _RESTARTRESPONSE,
  '__module__' : 'restart_pb2'
  # @@protoc_insertion_point(class_scope:device.api.RestartResponse)
  })
_sym_db.RegisterMessage(RestartResponse)

UptimeGetResponse = _reflection.GeneratedProtocolMessageType('UptimeGetResponse', (_message.Message,), {
  'DESCRIPTOR' : _UPTIMEGETRESPONSE,
  '__module__' : 'restart_pb2'
  # @@protoc_insertion_point(class_scope:device.api.UptimeGetResponse)
  })
_sym_db.RegisterMessage(UptimeGetResponse)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\006deviceB\007restart'
  _RESTARTRESPONSE._serialized_start=30
  _RESTARTRESPONSE._serialized_end=158
  _RESTARTRESPONSE_RESULT._serialized_start=117
  _RESTARTRESPONSE_RESULT._serialized_end=158
  _UPTIMEGETRESPONSE._serialized_start=160
  _UPTIMEGETRESPONSE._serialized_end=195
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: identifydevice.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14identifydevice.proto\x12\nplcnet.api\"*\n\x13IdentifyDeviceStart\x12\x13\n\x0bmac_address\x18\x01 \x01(\t\")\n\x12IdentifyDeviceStop\x12\x13\n\x0bmac_address\x18\x01 \x01(\t\"\xc2\x01\n\x16IdentifyDeviceResponse\x12\x39\n\x06result\x18\x01 \x01(\x0e\x32).plcnet.api.IdentifyDeviceResponse.Result\"m\n\x06Result\x12\x0b\n\x07SUCCESS\x10\x00\x12\x13\n\x0fMACADDR_INVALID\x10\x01\x12\x13\n\x0fMACADDR_UNKNOWN\x10\x02\x12\x18\n\x13\x43OMMUNICATION_ERROR\x10\xfe\x01\x12\x12\n\rUNKNOWN_ERROR\x10\xff\x01\x42\x12\n\x06plcnetB\x08identityb\x06proto3')



_IDENTIFYDEVICESTART = DESCRIPTOR.message_types_by_name['IdentifyDeviceStart']
_IDENTIFYDEVICESTOP = DESCRIPTOR.message_types_by_name['IdentifyDeviceStop']
_IDENTIFYDEVICERESPONSE = DESCRIPTOR.message_types_by_name['IdentifyDeviceResponse']
_IDENTIFYDEVICERESPONSE_RESULT = _IDENTIFYDEVICERESPONSE.enum_types_by_name['Result']
IdentifyDeviceStart = _reflection.GeneratedProtocolMessageType('IdentifyDeviceStart', (_message.Message,), {
  'DESCRIPTOR' : _IDENTIFYDEVICESTART,
  '__module__' : 'identifydevice_pb2'
  # @@protoc_insertion_point(class_scope:plcnet.api.IdentifyDeviceStart)
  })
_sym_db.RegisterMessage(IdentifyDeviceStart)

IdentifyDeviceStop = _reflection.GeneratedProtocolMessageType('IdentifyDeviceStop', (_message.Message,), {
  'DESCRIPTOR' : _IDENTIFYDEVICESTOP,
  '__module__' : 'identifydevice_pb2'
  # @@protoc_insertion_point(class_scope:plcnet.api.IdentifyDeviceStop)
  })
_sym_db.RegisterMessage(IdentifyDeviceStop)

IdentifyDeviceResponse = _reflection.GeneratedProtocolMessageType('IdentifyDeviceResponse', (_message.Message,), {
  'DESCRIPTOR' : _IDENTIFYDEVICERESPONSE,
  '__module__' : 'identifydevice_pb2'
  # @@protoc_insertion_point(class_scope:plcnet.api.IdentifyDeviceResponse)
  })
_sym_db.RegisterMessage(IdentifyDeviceResponse)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\006plcnetB\010identity'
  _IDENTIFYDEVICESTART._serialized_start=36
  _IDENTIFYDEVICESTART._serialized_end=78
  _IDENTIFYDEVICESTOP._serialized_start=80
  _IDENTIFYDEVICESTOP._serialized_end=121
  _IDENTIFYDEVICERESPONSE._serialized_start=124
  _IDENTIFYDEVICERESPONSE._serialized_end=318
  _IDENTIFYDEVICERESPONSE_RESULT._serialized_start=209
  _IDENTIFYDEVICERESPONSE_RESULT._serialized_end=318
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class RestartResponse(google.protobuf.message.Message):
    """
    Response to calling the "Restart" endpoint to perform a restart of a devolo device.
    The result code shows only the success of the triggering.
    The current uptime of the device is returned so that it can be compared
    to an uptime retrieved at a later time to check for a successful restart.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class _Result:
        ValueType = typing.NewType('ValueType', builtins.int)
        V: typing_extensions.TypeAlias = ValueType
    class _ResultEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[RestartResponse._Result.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        SUCCESS: RestartResponse._Result.ValueType  # 0
        """The restart can and will be triggered."""

        UNKNOWN_ERROR: RestartResponse._Result.ValueType  # 255
        """For some unknown error a restart cannot be triggered at this time."""

    class Result(_Result, metaclass=_ResultEnumTypeWrapper):
        pass

    SUCCESS: RestartResponse.Result.ValueType  # 0
    """The restart can and will be triggered."""

    UNKNOWN_ERROR: RestartResponse.Result.ValueType  # 255
    """For some unknown error a restart cannot be triggered at this time."""


    RESULT_FIELD_NUMBER: builtins.int
    UPTIME_FIELD_NUMBER: builtins.int
    result: global___RestartResponse.Result.ValueType
    """SUCCESS, if restart can and will be triggered"""

    uptime: builtins.int
    """Current uptime of the device. No unit is attached to the value."""

    def __init__(self,
        *,
        result: global___RestartResponse.Result.ValueType = ...,
        uptime: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["result",b"result","uptime",b"uptime"]) -> None: ...
global___RestartResponse = RestartResponse

class UptimeGetResponse(google.protobuf.message.Message):
    """
    Response to calling the "UptimeGet" endpoint.
    The current uptime of the device is returned.
    No unit (like seconds or millisenconds or ticks) is attached to the value.
    You can only use the uptime value as a strict monotonically increasing number,
    that increases in regular intervals since the device started.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    UPTIME_FIELD_NUMBER: builtins.int
    uptime: builtins.int
    """Current uptime of the device. No unit is attached to the value."""

    def __init__(self,
        *,
        uptime: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["uptime",b"uptime"]) -> None: ...
global___UptimeGetResponse = UptimeGetResponse
